@@ -24,7 +24,11 @@
 #include <unistd.h>
 #include <curl/curl.h>
 
+#include "datatypes.h"
+#include "ch_conf.h"
+#include "ch_interface.h"
 #include "ch_monitor.h"
+#include "domain_interface.h"
 #include "viralloc.h"
 #include "vircommand.h"
 #include "virerror.h"
@@ -85,27 +89,71 @@ virCHMonitorBuildCPUJson(virJSONValue *content, virDomainDef *vmdef)
 }
 
 static int
-virCHMonitorBuildPTYJson(virJSONValue *content, virDomainDef *vmdef)
+virCHMonitorBuildConsoleJson(virJSONValue *content,
+                             virDomainDef *vmdef)
 {
-    if (vmdef->nconsoles) {
-        g_autoptr(virJSONValue) pty = virJSONValueNewObject();
-        if (virJSONValueObjectAppendString(pty, "mode", "Pty") < 0)
+    g_autoptr(virJSONValue) console = virJSONValueNewObject();
+    g_autoptr(virJSONValue) serial = virJSONValueNewObject();
+
+    if (vmdef->nconsoles &&
+        vmdef->consoles[0]->source->type == VIR_DOMAIN_CHR_TYPE_PTY) {
+        if (virJSONValueObjectAppendString(console, "mode", "Pty") < 0)
             return -1;
-        if (virJSONValueObjectAppend(content, "console", &pty) < 0)
+        if (virJSONValueObjectAppend(content, "console", &console) < 0)
             return -1;
     }
 
     if (vmdef->nserials) {
-        g_autoptr(virJSONValue) pty = virJSONValueNewObject();
-        if (virJSONValueObjectAppendString(pty, "mode", "Pty") < 0)
-            return -1;
-        if (virJSONValueObjectAppend(content, "serial", &pty) < 0)
+        if (vmdef->serials[0]->source->type == VIR_DOMAIN_CHR_TYPE_PTY) {
+            if (virJSONValueObjectAppendString(serial, "mode", "Pty") < 0)
+                return -1;
+        } else if (vmdef->serials[0]->source->type == VIR_DOMAIN_CHR_TYPE_UNIX) {
+            if (virJSONValueObjectAppendString(serial, "mode", "Socket") < 0)
+                return -1;
+            if (virJSONValueObjectAppendString(serial,
+                                               "socket",
+                                               vmdef->serials[0]->source->data.file.path) < 0)
+                return -1;
+        }
+
+        if (virJSONValueObjectAppend(content, "serial", &serial) < 0)
             return -1;
     }
 
     return 0;
 }
 
+static int
+virCHMonitorBuildPayloadJson(virJSONValue *content, virDomainDef *vmdef)
+{
+    g_autoptr(virJSONValue) payload = virJSONValueNewObject();
+
+
+    if (vmdef->os.kernel == NULL) {
+        virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                       _("Kernel image path in this domain is not defined"));
+        return -1;
+    } else {
+        if (virJSONValueObjectAppendString(payload, "kernel", vmdef->os.kernel) < 0)
+            return -1;
+    }
+
+    if (vmdef->os.cmdline) {
+        if (virJSONValueObjectAppendString(payload, "cmdline", vmdef->os.cmdline) < 0)
+            return -1;
+    }
+
+    if (vmdef->os.initrd != NULL) {
+        if (virJSONValueObjectAppendString(payload, "initramfs", vmdef->os.initrd) < 0)
+            return -1;
+    }
+
+    if (virJSONValueObjectAppend(content, "payload", &payload) < 0)
+    return -1;
+
+    return 0;
+}
+
 static int
 virCHMonitorBuildKernelRelatedJson(virJSONValue *content, virDomainDef *vmdef)
 {
@@ -197,6 +245,7 @@ virCHMonitorBuildDiskJson(virJSONValue *disks, virDomainDiskDef *diskdef)
     case VIR_STORAGE_TYPE_VOLUME:
     case VIR_STORAGE_TYPE_NVME:
     case VIR_STORAGE_TYPE_VHOST_USER:
+    case VIR_STORAGE_TYPE_VHOST_VDPA:
     case VIR_STORAGE_TYPE_LAST:
     default:
         virReportEnumRangeError(virStorageType, diskdef->src->type);
@@ -226,151 +275,95 @@ virCHMonitorBuildDisksJson(virJSONValue *content, virDomainDef *vmdef)
     return 0;
 }
 
-static int
-virCHMonitorBuildNetJson(virJSONValue *nets,
-                         virDomainNetDef *netdef,
-                         size_t *nnicindexes,
-                         int **nicindexes)
+/**
+ * virCHMonitorBuildNetJson:
+ * @net: pointer to a guest network definition
+ * @jsonstr: returned network json
+ *
+ * Build net json to send to CH
+ * Returns 0 on success or -1 in case of error
+ */
+int
+virCHMonitorBuildNetJson(virDomainNetDef *net, char **jsonstr)
 {
-    virDomainNetType netType = virDomainNetGetActualType(netdef);
     char macaddr[VIR_MAC_STRING_BUFLEN];
-    g_autoptr(virJSONValue) net = NULL;
+    g_autoptr(virJSONValue) net_json = virJSONValueNewObject();
+    virDomainNetType actualType = virDomainNetGetActualType(net);
 
-    // check net type at first
-    net = virJSONValueNewObject();
+    if (actualType == VIR_DOMAIN_NET_TYPE_ETHERNET &&
+        net->guestIP.nips == 1) {
+        const virNetDevIPAddr *ip;
+        g_autofree char *addr = NULL;
+        virSocketAddr netmask;
+        g_autofree char *netmaskStr = NULL;
 
-    switch (netType) {
-        case VIR_DOMAIN_NET_TYPE_ETHERNET:
-            if (netdef->guestIP.nips == 1) {
-                const virNetDevIPAddr *ip = netdef->guestIP.ips[0];
-                g_autofree char *addr = NULL;
-                virSocketAddr netmask;
-                g_autofree char *netmaskStr = NULL;
+        ip = net->guestIP.ips[0];
 
-                if (!(addr = virSocketAddrFormat(&ip->address)))
-                    return -1;
-                if (virJSONValueObjectAppendString(net, "ip", addr) < 0)
-                    return -1;
+        if (!(addr = virSocketAddrFormat(&ip->address)))
+            return -1;
 
-                if (virSocketAddrPrefixToNetmask(ip->prefix, &netmask, AF_INET) < 0) {
-                    virReportError(VIR_ERR_INTERNAL_ERROR,
-                                   _("Failed to translate net prefix %1$d to netmask"),
-                                   ip->prefix);
-                    return -1;
-                }
-                if (!(netmaskStr = virSocketAddrFormat(&netmask)))
-                    return -1;
-                if (virJSONValueObjectAppendString(net, "mask", netmaskStr) < 0)
-                    return -1;
-            } else if (netdef->guestIP.nips > 1) {
-                virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
-                               _("ethernet type supports a single guest ip"));
-            }
+        if (virJSONValueObjectAppendString(net_json, "ip", addr) < 0)
+            return -1;
 
-            /* network and bridge use a tap device, and direct uses a
-             * macvtap device
-             */
-            if (nicindexes && nnicindexes && netdef->ifname) {
-                int nicindex = 0;
+        if (virSocketAddrPrefixToNetmask(ip->prefix, &netmask, AF_INET) < 0) {
+            virReportError(VIR_ERR_INTERNAL_ERROR,
+                           _("Failed to translate net prefix %1$d to netmask"),
+                           ip->prefix);
+            return -1;
+        }
 
-                if (virNetDevGetIndex(netdef->ifname, &nicindex) < 0)
-                    return -1;
+        if (!(netmaskStr = virSocketAddrFormat(&netmask)))
+            return -1;
 
-                VIR_APPEND_ELEMENT(*nicindexes, *nnicindexes, nicindex);
-            }
-            break;
-        case VIR_DOMAIN_NET_TYPE_VHOSTUSER:
-            if ((virDomainChrType)netdef->data.vhostuser->type != VIR_DOMAIN_CHR_TYPE_UNIX) {
-                virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
-                           _("vhost_user type support UNIX socket in this CH"));
-                return -1;
-            } else {
-                if (virJSONValueObjectAppendString(net, "vhost_socket", netdef->data.vhostuser->data.nix.path) < 0)
-                    return -1;
-                if (virJSONValueObjectAppendBoolean(net, "vhost_user", true) < 0)
-                    return -1;
-            }
-            break;
-        case VIR_DOMAIN_NET_TYPE_BRIDGE:
-        case VIR_DOMAIN_NET_TYPE_NETWORK:
-        case VIR_DOMAIN_NET_TYPE_DIRECT:
-        case VIR_DOMAIN_NET_TYPE_USER:
-        case VIR_DOMAIN_NET_TYPE_SERVER:
-        case VIR_DOMAIN_NET_TYPE_CLIENT:
-        case VIR_DOMAIN_NET_TYPE_MCAST:
-        case VIR_DOMAIN_NET_TYPE_INTERNAL:
-        case VIR_DOMAIN_NET_TYPE_HOSTDEV:
-        case VIR_DOMAIN_NET_TYPE_UDP:
-        case VIR_DOMAIN_NET_TYPE_VDPA:
-        case VIR_DOMAIN_NET_TYPE_NULL:
-        case VIR_DOMAIN_NET_TYPE_VDS:
-        case VIR_DOMAIN_NET_TYPE_LAST:
-        default:
-            virReportEnumRangeError(virDomainNetType, netType);
+        if (virJSONValueObjectAppendString(net_json, "mask", netmaskStr) < 0)
             return -1;
     }
 
-    if (netdef->ifname != NULL) {
-        if (virJSONValueObjectAppendString(net, "tap", netdef->ifname) < 0)
-            return -1;
-    }
-    if (virJSONValueObjectAppendString(net, "mac", virMacAddrFormat(&netdef->mac, macaddr)) < 0)
+    if (virJSONValueObjectAppendString(net_json, "mac",
+                                       virMacAddrFormat(&net->mac, macaddr)) < 0)
         return -1;
 
-
-    if (netdef->virtio != NULL) {
-        if (netdef->virtio->iommu == VIR_TRISTATE_SWITCH_ON) {
-            if (virJSONValueObjectAppendBoolean(net, "iommu", true) < 0)
+    if (net->virtio != NULL) {
+        if (net->virtio->iommu == VIR_TRISTATE_SWITCH_ON) {
+            if (virJSONValueObjectAppendBoolean(net_json, "iommu", true) < 0)
                 return -1;
         }
     }
-    if (netdef->driver.virtio.queues) {
-        if (virJSONValueObjectAppendNumberInt(net, "num_queues", netdef->driver.virtio.queues) < 0)
+
+    /* Cloud-Hypervisor expects number of queues. 1 for rx and 1 for tx.
+     * Multiply queue pairs by 2 to provide total number of queues to CH
+     */
+    if (net->driver.virtio.queues) {
+        if (virJSONValueObjectAppendNumberInt(net_json, "num_queues",
+                                              2 * net->driver.virtio.queues) < 0)
             return -1;
     }
 
-    if (netdef->driver.virtio.rx_queue_size || netdef->driver.virtio.tx_queue_size) {
-        if (netdef->driver.virtio.rx_queue_size != netdef->driver.virtio.tx_queue_size) {
+    if (net->driver.virtio.rx_queue_size || net->driver.virtio.tx_queue_size) {
+        if (net->driver.virtio.rx_queue_size !=
+            net->driver.virtio.tx_queue_size) {
             virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
-               _("virtio rx_queue_size option %1$d is not same with tx_queue_size %2$d"),
-               netdef->driver.virtio.rx_queue_size,
-               netdef->driver.virtio.tx_queue_size);
+                           _("virtio rx_queue_size option %1$d is not same with tx_queue_size %2$d"),
+                           net->driver.virtio.rx_queue_size,
+                           net->driver.virtio.tx_queue_size);
             return -1;
         }
-        if (virJSONValueObjectAppendNumberInt(net, "queue_size", netdef->driver.virtio.rx_queue_size) < 0)
+        if (virJSONValueObjectAppendNumberInt(net_json, "queue_size",
+                                              net->driver.virtio.rx_queue_size) < 0)
             return -1;
     }
 
-    if (virJSONValueArrayAppend(nets, &net) < 0)
+    if (net->mtu) {
+        if (virJSONValueObjectAppendNumberInt(net_json, "mtu", net->mtu) < 0)
+            return -1;
+    }
+
+    if (!(*jsonstr = virJSONValueToString(net_json, false)))
         return -1;
 
     return 0;
 }
 
-static int
-virCHMonitorBuildNetsJson(virJSONValue *content,
-                          virDomainDef *vmdef,
-                          size_t *nnicindexes,
-                          int **nicindexes)
-{
-    g_autoptr(virJSONValue) nets = NULL;
-    size_t i;
-
-    if (vmdef->nnets > 0) {
-        nets = virJSONValueNewArray();
-
-        for (i = 0; i < vmdef->nnets; i++) {
-            if (virCHMonitorBuildNetJson(nets, vmdef->nets[i],
-                                         nnicindexes, nicindexes) < 0)
-                return -1;
-        }
-        if (virJSONValueObjectAppend(content, "net", &nets) < 0)
-            return -1;
-    }
-
-    return 0;
-}
-
 static int
 virCHMonitorBuildDeviceJson(virJSONValue *devices,
                             virDomainHostdevDef *hostdevdef)
@@ -424,10 +417,8 @@ virCHMonitorBuildDevicesJson(virJSONValue *content,
 }
 
 static int
-virCHMonitorBuildVMJson(virDomainDef *vmdef,
-                        char **jsonstr,
-                        size_t *nnicindexes,
-                        int **nicindexes)
+virCHMonitorBuildVMJson(virCHDriver *driver, virDomainDef *vmdef,
+                        char **jsonstr)
 {
     g_autoptr(virJSONValue) content = virJSONValueNewObject();
 
@@ -437,7 +428,7 @@ virCHMonitorBuildVMJson(virDomainDef *vmdef,
         return -1;
     }
 
-    if (virCHMonitorBuildPTYJson(content, vmdef) < 0)
+    if (virCHMonitorBuildConsoleJson(content, vmdef) < 0)
         return -1;
 
     if (virCHMonitorBuildCPUJson(content, vmdef) < 0)
@@ -446,16 +437,17 @@ virCHMonitorBuildVMJson(virDomainDef *vmdef,
     if (virCHMonitorBuildMemoryJson(content, vmdef) < 0)
         return -1;
 
-    if (virCHMonitorBuildKernelRelatedJson(content, vmdef) < 0)
-        return -1;
+    if (virBitmapIsBitSet(driver->chCaps, CH_KERNEL_API_DEPRCATED)) {
+        if (virCHMonitorBuildPayloadJson(content, vmdef) < 0)
+            return -1;
+    } else if (virCHMonitorBuildKernelRelatedJson(content, vmdef) < 0) {
+            return -1;
+    }
+
 
     if (virCHMonitorBuildDisksJson(content, vmdef) < 0)
         return -1;
 
-
-    if (virCHMonitorBuildNetsJson(content, vmdef, nnicindexes, nicindexes) < 0)
-        return -1;
-
     if (virCHMonitorBuildDevicesJson(content, vmdef) < 0)
         return -1;
 
@@ -465,10 +457,26 @@ virCHMonitorBuildVMJson(virDomainDef *vmdef,
     return 0;
 }
 
+static int
+virCHMonitorBuildKeyValueStringJson(char **jsonstr,
+                                    const char *key,
+                                    const char *value)
+{
+    g_autoptr(virJSONValue) content = virJSONValueNewObject();
+
+    if (virJSONValueObjectAppendString(content, key, value) < 0)
+        return -1;
+
+    if (!(*jsonstr = virJSONValueToString(content, false)))
+        return -1;
+
+    return 0;
+}
+
 static int
 chMonitorCreateSocket(const char *socket_path)
 {
-    struct sockaddr_un addr;
+    struct sockaddr_un addr = { 0 };
     socklen_t addrlen = sizeof(addr);
     int fd;
 
@@ -478,7 +486,6 @@ chMonitorCreateSocket(const char *socket_path)
         goto error;
     }
 
-    memset(&addr, 0, sizeof(addr));
     addr.sun_family = AF_UNIX;
     if (virStrcpyStatic(addr.sun_path, socket_path) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
@@ -522,10 +529,11 @@ chMonitorCreateSocket(const char *socket_path)
 }
 
 virCHMonitor *
-virCHMonitorNew(virDomainObj *vm, const char *socketdir)
+virCHMonitorNew(virDomainObj *vm, virCHDriverConfig *cfg)
 {
     g_autoptr(virCHMonitor) mon = NULL;
     g_autoptr(virCommand) cmd = NULL;
+    const char *socketdir = cfg->stateDir;
     int socket_fd = 0;
 
     if (virCHMonitorInitialize() < 0)
@@ -549,6 +557,13 @@ virCHMonitorNew(virDomainObj *vm, const char *socketdir)
         return NULL;
     }
 
+    if (g_mkdir_with_parents(cfg->saveDir, 0777) < 0) {
+        virReportSystemError(errno,
+                             _("Cannot create save directory '%1$s'"),
+                             cfg->saveDir);
+        return NULL;
+    }
+
     cmd = virCommandNew(vm->def->emulator);
     virCommandSetUmask(cmd, 0x002);
     socket_fd = chMonitorCreateSocket(mon->socketpath);
@@ -638,8 +653,7 @@ virCHMonitorCurlPerform(CURL *handle)
 
     if (responseCode < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
-                       _("curl_easy_getinfo(CURLINFO_RESPONSE_CODE) returned a "
-                         "negative response code"));
+                       _("curl_easy_getinfo(CURLINFO_RESPONSE_CODE) returned a negative response code"));
         return -1;
     }
 
@@ -841,9 +855,7 @@ virCHMonitorShutdownVMM(virCHMonitor *mon)
 }
 
 int
-virCHMonitorCreateVM(virCHMonitor *mon,
-                     size_t *nnicindexes,
-                     int **nicindexes)
+virCHMonitorCreateVM(virCHDriver *driver, virCHMonitor *mon)
 {
     g_autofree char *url = NULL;
     int responseCode = 0;
@@ -855,8 +867,7 @@ virCHMonitorCreateVM(virCHMonitor *mon,
     headers = curl_slist_append(headers, "Accept: application/json");
     headers = curl_slist_append(headers, "Content-Type: application/json");
 
-    if (virCHMonitorBuildVMJson(mon->vm->def, &payload,
-                                nnicindexes, nicindexes) != 0)
+    if (virCHMonitorBuildVMJson(driver, mon->vm->def, &payload) != 0)
         return -1;
 
     VIR_WITH_OBJECT_LOCK_GUARD(mon) {
@@ -909,6 +920,77 @@ virCHMonitorResumeVM(virCHMonitor *mon)
     return virCHMonitorPutNoContent(mon, URL_VM_RESUME);
 }
 
+static int
+virCHMonitorSaveRestoreVM(virCHMonitor *mon, const char *path, bool save)
+{
+    g_autofree char *url = NULL;
+    int responseCode = 0;
+    int ret = -1;
+    g_autofree char *payload = NULL;
+    g_autofree char *path_url = NULL;
+    struct curl_slist *headers = NULL;
+    struct curl_data data = {0};
+
+    if (save)
+        url = g_strdup_printf("%s/%s", URL_ROOT, URL_VM_SAVE);
+    else
+        url = g_strdup_printf("%s/%s", URL_ROOT, URL_VM_RESTORE);
+
+    headers = curl_slist_append(headers, "Accept: application/json");
+    headers = curl_slist_append(headers, "Content-Type: application/json");
+
+    path_url = g_strdup_printf("file://%s", path);
+    if (save) {
+        if (virCHMonitorBuildKeyValueStringJson(&payload, "destination_url", path_url) != 0)
+            return -1;
+    } else {
+        if (virCHMonitorBuildKeyValueStringJson(&payload, "source_url", path_url) != 0)
+            return -1;
+    }
+
+    VIR_WITH_OBJECT_LOCK_GUARD(mon) {
+        /* reset all options of a libcurl session handle at first */
+        curl_easy_reset(mon->handle);
+
+        curl_easy_setopt(mon->handle, CURLOPT_UNIX_SOCKET_PATH, mon->socketpath);
+        curl_easy_setopt(mon->handle, CURLOPT_URL, url);
+        curl_easy_setopt(mon->handle, CURLOPT_CUSTOMREQUEST, "PUT");
+        curl_easy_setopt(mon->handle, CURLOPT_HTTPHEADER, headers);
+        curl_easy_setopt(mon->handle, CURLOPT_POSTFIELDS, payload);
+        curl_easy_setopt(mon->handle, CURLOPT_WRITEFUNCTION, curl_callback);
+        curl_easy_setopt(mon->handle, CURLOPT_WRITEDATA, (void *)&data);
+
+        responseCode = virCHMonitorCurlPerform(mon->handle);
+    }
+
+    if (responseCode == 200 || responseCode == 204) {
+        ret = 0;
+    } else {
+        data.content = g_realloc(data.content, data.size + 1);
+        data.content[data.size] = 0;
+        virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                       data.content);
+        g_free(data.content);
+    }
+
+    /* reset the libcurl handle to avoid leaking a stack pointer to data */
+    curl_easy_reset(mon->handle);
+    curl_slist_free_all(headers);
+    return ret;
+}
+
+int
+virCHMonitorSaveVM(virCHMonitor *mon, const char *to)
+{
+    return virCHMonitorSaveRestoreVM(mon, to, true);
+}
+
+int
+virCHMonitorRestoreVM(virCHMonitor *mon, const char *from)
+{
+    return virCHMonitorSaveRestoreVM(mon, from, false);
+}
+
 /**
  * virCHMonitorGetInfo:
  * @mon: Pointer to the monitor
@@ -939,7 +1021,7 @@ virCHMonitorGetIOThreads(virCHMonitor *mon,
                          virDomainIOThreadInfo ***iothreads)
 {
     size_t nthreads = 0;
-    size_t niothreads = 0;
+    int niothreads = 0;
     int thd_index;
     virDomainIOThreadInfo **iothreadinfolist = NULL;
     virDomainIOThreadInfo *iothreadinfo = NULL;
@@ -971,7 +1053,7 @@ virCHMonitorGetIOThreads(virCHMonitor *mon,
         }
     }
 
-    VIR_DEBUG("niothreads = %ld", niothreads);
+    VIR_DEBUG("niothreads = %d", niothreads);
     *iothreads = g_steal_pointer(&iothreadinfolist);
     return niothreads;
 
