@@ -20,26 +20,36 @@
  */
 #include <config.h>
 
-#include <errno.h>
 #include <dirent.h>
 #include <libudev.h>
 
+#include "virlog.h"
 #include "virerror.h"
 #include "virfile.h"
-#include "c-ctype.h"
 #include "datatypes.h"
-#include "domain_conf.h"
 #include "interface_driver.h"
 #include "interface_conf.h"
 #include "viralloc.h"
 #include "virstring.h"
+#include "virpidfile.h"
 #include "viraccessapicheck.h"
+#include "virinterfaceobj.h"
 #include "virnetdev.h"
+#include "virutil.h"
+
+#include "configmake.h"
 
 #define VIR_FROM_THIS VIR_FROM_INTERFACE
 
+VIR_LOG_INIT("interface.interface_backend_udev");
+
 struct udev_iface_driver {
     struct udev *udev;
+    /* pid file FD, ensures two copies of the driver can't use the same root */
+    int lockFD;
+
+    char *stateDir;
+    bool privileged;
 };
 
 typedef enum {
@@ -78,20 +88,12 @@ udevGetMinimalDefForDevice(struct udev_device *dev)
     virInterfaceDef *def;
 
     /* Allocate our interface definition structure */
-    if (VIR_ALLOC(def) < 0)
-        return NULL;
+    def = g_new0(virInterfaceDef, 1);
 
-    if (VIR_STRDUP(def->name, udev_device_get_sysname(dev)) < 0)
-        goto cleanup;
-
-    if (VIR_STRDUP(def->mac, udev_device_get_sysattr_value(dev, "address")) < 0)
-        goto cleanup;
+    def->name = g_strdup(udev_device_get_sysname(dev));
+    def->mac = g_strdup(udev_device_get_sysattr_value(dev, "address"));
 
     return def;
-
- cleanup:
-    virInterfaceDefFree(def);
-    return NULL;
 }
 
 
@@ -135,56 +137,21 @@ udevGetDevices(struct udev *udev, virUdevStatus status)
     return enumerate;
 }
 
-static int
-udevNumOfInterfacesByStatus(virConnectPtr conn, virUdevStatus status,
-                            virInterfaceObjListFilter filter)
-{
-    struct udev *udev = udev_ref(driver->udev);
-    struct udev_enumerate *enumerate = NULL;
-    struct udev_list_entry *devices;
-    struct udev_list_entry *dev_entry;
-    int count = 0;
-
-    enumerate = udevGetDevices(udev, status);
-
-    if (!enumerate) {
-        virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("failed to get number of %s interfaces on host"),
-                       virUdevStatusString(status));
-        count = -1;
-        goto cleanup;
-    }
-
-    /* Do the scan to load up the enumeration */
-    udev_enumerate_scan_devices(enumerate);
-
-    /* Get a list we can walk */
-    devices = udev_enumerate_get_list_entry(enumerate);
-
-    /* For each item so we can count */
-    udev_list_entry_foreach(dev_entry, devices) {
-        struct udev_device *dev;
-        const char *path;
-        virInterfaceDefPtr def;
-
-        path = udev_list_entry_get_name(dev_entry);
-        dev = udev_device_new_from_syspath(udev, path);
-
-        def = udevGetMinimalDefForDevice(dev);
-        if (filter(conn, def))
-            count++;
-        udev_device_unref(dev);
-        virInterfaceDefFree(def);
-    }
-
- cleanup:
-    if (enumerate)
-        udev_enumerate_unref(enumerate);
-    udev_unref(udev);
-
-    return count;
-}
 
+/**
+ * udevListInterfacesByStatus:
+ *
+ * @conn: connection object
+ * @names: optional pointer to array to be filled with interface names
+ * @names_len: size of @names
+ * @status: status of interfaces to be listed
+ * @filter: ACL filter function
+ *
+ * Lists interfaces with status matching @status filling them into @names (if
+ * non-NULL) and returns the number of such interfaces.
+ *
+ * In case of an error -1 is returned and no interfaces are filled into @names.
+ */
 static int
 udevListInterfacesByStatus(virConnectPtr conn,
                            char **const names,
@@ -202,9 +169,10 @@ udevListInterfacesByStatus(virConnectPtr conn,
 
     if (!enumerate) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("failed to get list of %s interfaces on host"),
+                       _("failed to get list of %1$s interfaces on host"),
                        virUdevStatusString(status));
-        goto error;
+        udev_unref(udev);
+        return -1;
     }
 
     /* Do the scan to load up the enumeration */
@@ -217,52 +185,47 @@ udevListInterfacesByStatus(virConnectPtr conn,
     udev_list_entry_foreach(dev_entry, devices) {
         struct udev_device *dev;
         const char *path;
-        virInterfaceDefPtr def;
+        const char *name;
+        g_autoptr(virInterfaceDef) def = NULL;
 
         /* Ensure we won't exceed the size of our array */
-        if (count > names_len)
+        if (names && count >= names_len)
             break;
 
         path = udev_list_entry_get_name(dev_entry);
         dev = udev_device_new_from_syspath(udev, path);
 
+        if (!(name = udev_device_get_sysname(dev))) {
+            /* Name can be NULL in case when the interface is being unbound
+             * from the driver. The list API requires names to be present */
+            VIR_DEBUG("Skipping interface '%s', name == NULL", path);
+            continue;
+        }
+
         def = udevGetMinimalDefForDevice(dev);
         if (filter(conn, def)) {
-            if (VIR_STRDUP(names[count], udev_device_get_sysname(dev)) < 0) {
-                udev_device_unref(dev);
-                virInterfaceDefFree(def);
-                goto error;
-            }
+            if (names)
+                names[count] = g_strdup(name);
             count++;
         }
         udev_device_unref(dev);
-        virInterfaceDefFree(def);
     }
 
     udev_enumerate_unref(enumerate);
     udev_unref(udev);
 
     return count;
-
- error:
-    if (enumerate)
-        udev_enumerate_unref(enumerate);
-    udev_unref(udev);
-
-    for (names_len = 0; names_len < count; names_len++)
-        VIR_FREE(names[names_len]);
-
-    return -1;
 }
 
+
 static int
 udevConnectNumOfInterfaces(virConnectPtr conn)
 {
     if (virConnectNumOfInterfacesEnsureACL(conn) < 0)
         return -1;
 
-    return udevNumOfInterfacesByStatus(conn, VIR_UDEV_IFACE_ACTIVE,
-                                       virConnectNumOfInterfacesCheckACL);
+    return udevListInterfacesByStatus(conn, NULL, 0, VIR_UDEV_IFACE_ACTIVE,
+                                      virConnectNumOfInterfacesCheckACL);
 }
 
 static int
@@ -284,8 +247,8 @@ udevConnectNumOfDefinedInterfaces(virConnectPtr conn)
     if (virConnectNumOfDefinedInterfacesEnsureACL(conn) < 0)
         return -1;
 
-    return udevNumOfInterfacesByStatus(conn, VIR_UDEV_IFACE_INACTIVE,
-                                       virConnectNumOfDefinedInterfacesCheckACL);
+    return udevListInterfacesByStatus(conn, NULL, 0, VIR_UDEV_IFACE_INACTIVE,
+                                      virConnectNumOfDefinedInterfacesCheckACL);
 }
 
 static int
@@ -313,10 +276,9 @@ udevConnectListAllInterfaces(virConnectPtr conn,
     struct udev_list_entry *dev_entry;
     virInterfacePtr *ifaces_list = NULL;
     virInterfacePtr iface_obj;
-    int tmp_count;
     int count = 0;
     int status = 0;
-    int ret;
+    int ret = -1;
 
     virCheckFlags(VIR_CONNECT_LIST_INTERFACES_FILTERS_ACTIVE, -1);
 
@@ -331,9 +293,8 @@ udevConnectListAllInterfaces(virConnectPtr conn,
 
     if (!enumerate) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("failed to get list of %s interfaces on host"),
+                       _("failed to get list of %1$s interfaces on host"),
                        virUdevStatusString(status));
-        ret = -1;
         goto cleanup;
     }
 
@@ -355,10 +316,8 @@ udevConnectListAllInterfaces(virConnectPtr conn,
     }
 
     /* If we're asked for the ifaces then alloc up memory */
-    if (ifaces && VIR_ALLOC_N(ifaces_list, count + 1) < 0) {
-        ret = -1;
-        goto cleanup;
-    }
+    if (ifaces)
+        ifaces_list = g_new0(virInterfacePtr, count + 1);
 
     /* Get a list we can walk */
     devices = udev_enumerate_get_list_entry(enumerate);
@@ -372,21 +331,28 @@ udevConnectListAllInterfaces(virConnectPtr conn,
         const char *path;
         const char *name;
         const char *macaddr;
-        virInterfaceDefPtr def;
+        g_autoptr(virInterfaceDef) def = NULL;
 
-        path = udev_list_entry_get_name(dev_entry);
-        dev = udev_device_new_from_syspath(udev, path);
-        name = udev_device_get_sysname(dev);
+        if (!(path = udev_list_entry_get_name(dev_entry))) {
+            VIR_DEBUG("Skipping interface, path == NULL");
+            continue;
+        }
+        if (!(dev = udev_device_new_from_syspath(udev, path))) {
+            VIR_DEBUG("Skipping interface '%s', dev == NULL", path);
+            continue;
+        }
+        if (!(name = udev_device_get_sysname(dev))) {
+            VIR_DEBUG("Skipping interface '%s', name == NULL", path);
+            continue;
+        }
         macaddr = udev_device_get_sysattr_value(dev, "address");
-        status = STREQ(udev_device_get_sysattr_value(dev, "operstate"), "up");
+        status = STREQ_NULLABLE(udev_device_get_sysattr_value(dev, "operstate"), "up");
 
         def = udevGetMinimalDefForDevice(dev);
         if (!virConnectListAllInterfacesCheckACL(conn, def)) {
             udev_device_unref(dev);
-            virInterfaceDefFree(def);
             continue;
         }
-        virInterfaceDefFree(def);
 
         /* Filter the results */
         if (MATCH(VIR_CONNECT_LIST_INTERFACES_FILTERS_ACTIVE) &&
@@ -410,9 +376,8 @@ udevConnectListAllInterfaces(virConnectPtr conn,
 
     /* Trim the array to its final size */
     if (ifaces) {
-        ignore_value(VIR_REALLOC_N(ifaces_list, count + 1));
-        *ifaces = ifaces_list;
-        ifaces_list = NULL;
+        VIR_REALLOC_N(ifaces_list, count + 1);
+        *ifaces = g_steal_pointer(&ifaces_list);
     }
 
     return count;
@@ -421,14 +386,6 @@ udevConnectListAllInterfaces(virConnectPtr conn,
     if (enumerate)
         udev_enumerate_unref(enumerate);
     udev_unref(udev);
-
-    if (ifaces) {
-        for (tmp_count = 0; tmp_count < count; tmp_count++)
-            virObjectUnref(ifaces_list[tmp_count]);
-    }
-
-    VIR_FREE(ifaces_list);
-
     return ret;
 
 }
@@ -439,13 +396,13 @@ udevInterfaceLookupByName(virConnectPtr conn, const char *name)
     struct udev *udev = udev_ref(driver->udev);
     struct udev_device *dev;
     virInterfacePtr ret = NULL;
-    virInterfaceDefPtr def = NULL;
+    g_autoptr(virInterfaceDef) def = NULL;
 
     /* get a device reference based on the device name */
     dev = udev_device_new_from_subsystem_sysname(udev, "net", name);
     if (!dev) {
         virReportError(VIR_ERR_NO_INTERFACE,
-                       _("couldn't find interface named '%s'"),
+                       _("couldn't find interface named '%1$s'"),
                        name);
         goto cleanup;
     }
@@ -461,7 +418,6 @@ udevInterfaceLookupByName(virConnectPtr conn, const char *name)
 
  cleanup:
     udev_unref(udev);
-    virInterfaceDefFree(def);
 
     return ret;
 }
@@ -473,14 +429,14 @@ udevInterfaceLookupByMACString(virConnectPtr conn, const char *macstr)
     struct udev_enumerate *enumerate = NULL;
     struct udev_list_entry *dev_entry;
     struct udev_device *dev;
-    virInterfaceDefPtr def = NULL;
+    g_autoptr(virInterfaceDef) def = NULL;
     virInterfacePtr ret = NULL;
 
     enumerate = udevGetDevices(udev, VIR_UDEV_IFACE_ALL);
 
     if (!enumerate) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("failed to lookup interface with MAC address '%s'"),
+                       _("failed to lookup interface with MAC address '%1$s'"),
                        macstr);
         goto cleanup;
     }
@@ -497,7 +453,7 @@ udevInterfaceLookupByMACString(virConnectPtr conn, const char *macstr)
     /* Check that we got something back */
     if (!dev_entry) {
         virReportError(VIR_ERR_NO_INTERFACE,
-                       _("couldn't find interface with MAC address '%s'"),
+                       _("couldn't find interface with MAC address '%1$s'"),
                        macstr);
         goto cleanup;
     }
@@ -505,7 +461,7 @@ udevInterfaceLookupByMACString(virConnectPtr conn, const char *macstr)
     /* Check that we didn't get multiple items back */
     if (udev_list_entry_get_next(dev_entry)) {
         virReportError(VIR_ERR_MULTIPLE_INTERFACES,
-                       _("the MAC address '%s' matches multiple interfaces"),
+                       _("the MAC address '%1$s' matches multiple interfaces"),
                        macstr);
         goto cleanup;
     }
@@ -525,7 +481,6 @@ udevInterfaceLookupByMACString(virConnectPtr conn, const char *macstr)
     if (enumerate)
         udev_enumerate_unref(enumerate);
     udev_unref(udev);
-    virInterfaceDefFree(def);
 
     return ret;
 }
@@ -569,8 +524,8 @@ udevBridgeScanDirFilter(const struct dirent *entry)
      * vnet%d. Improvements to this check are welcome.
      */
     if (strlen(entry->d_name) >= 5) {
-        if (STRPREFIX(entry->d_name, VIR_NET_GENERATED_PREFIX) &&
-            c_isdigit(entry->d_name[4]))
+        if (STRPREFIX(entry->d_name, VIR_NET_GENERATED_VNET_PREFIX) &&
+            g_ascii_isdigit(entry->d_name[4]))
             return 0;
     }
 
@@ -580,7 +535,7 @@ udevBridgeScanDirFilter(const struct dirent *entry)
 
 static int
 ATTRIBUTE_NONNULL(1) ATTRIBUTE_NONNULL(2) ATTRIBUTE_NONNULL(3)
-ATTRIBUTE_NONNULL(4) ATTRIBUTE_RETURN_CHECK
+ATTRIBUTE_NONNULL(4) G_GNUC_WARN_UNUSED_RESULT
 udevGetIfaceDefBond(struct udev *udev,
                     struct udev_device *dev,
                     const char *name,
@@ -601,12 +556,12 @@ udevGetIfaceDefBond(struct udev *udev,
     tmp_str = udev_device_get_sysattr_value(dev, "bonding/downdelay");
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not retrieve 'bonding/downdelay' for '%s'"), name);
+                _("Could not retrieve 'bonding/downdelay' for '%1$s'"), name);
         goto error;
     }
     if (virStrToLong_i(tmp_str, NULL, 10, &tmp_int) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not parse 'bonding/downdelay' '%s' for '%s'"),
+                _("Could not parse 'bonding/downdelay' '%1$s' for '%2$s'"),
                 tmp_str, name);
         goto error;
     }
@@ -615,12 +570,12 @@ udevGetIfaceDefBond(struct udev *udev,
     tmp_str = udev_device_get_sysattr_value(dev, "bonding/updelay");
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not retrieve 'bonding/updelay' for '%s'"), name);
+                _("Could not retrieve 'bonding/updelay' for '%1$s'"), name);
         goto error;
     }
     if (virStrToLong_i(tmp_str, NULL, 10, &tmp_int) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not parse 'bonding/updelay' '%s' for '%s'"),
+                _("Could not parse 'bonding/updelay' '%1$s' for '%2$s'"),
                 tmp_str, name);
         goto error;
     }
@@ -629,12 +584,12 @@ udevGetIfaceDefBond(struct udev *udev,
     tmp_str = udev_device_get_sysattr_value(dev, "bonding/miimon");
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not retrieve 'bonding/miimon' for '%s'"), name);
+                _("Could not retrieve 'bonding/miimon' for '%1$s'"), name);
         goto error;
     }
     if (virStrToLong_i(tmp_str, NULL, 10, &tmp_int) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not parse 'bonding/miimon' '%s' for '%s'"),
+                _("Could not parse 'bonding/miimon' '%1$s' for '%2$s'"),
                 tmp_str, name);
         goto error;
     }
@@ -643,12 +598,12 @@ udevGetIfaceDefBond(struct udev *udev,
     tmp_str = udev_device_get_sysattr_value(dev, "bonding/arp_interval");
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not retrieve 'bonding/arp_interval' for '%s'"), name);
+                _("Could not retrieve 'bonding/arp_interval' for '%1$s'"), name);
         goto error;
     }
     if (virStrToLong_i(tmp_str, NULL, 10, &tmp_int) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not parse 'bonding/arp_interval' '%s' for '%s'"),
+                _("Could not parse 'bonding/arp_interval' '%1$s' for '%2$s'"),
                 tmp_str, name);
         goto error;
     }
@@ -656,30 +611,30 @@ udevGetIfaceDefBond(struct udev *udev,
 
     /* bonding/mode is in the format: "balance-rr 0" so we find the
      * space and increment the pointer to get the number and convert
-     * it to an interger. libvirt uses 1 through 7 while the raw
+     * it to an integer. libvirt uses 1 through 7 while the raw
      * number is 0 through 6 so increment it by 1.
      */
     tmp_str = udev_device_get_sysattr_value(dev, "bonding/mode");
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not retrieve 'bonding/mode' for '%s'"), name);
+                _("Could not retrieve 'bonding/mode' for '%1$s'"), name);
         goto error;
     }
     tmp_str = strchr(tmp_str, ' ');
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Invalid format for 'bonding/mode' for '%s'"), name);
+                _("Invalid format for 'bonding/mode' for '%1$s'"), name);
         goto error;
     }
     if (strlen(tmp_str) < 2) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Unable to find correct value in 'bonding/mode' for '%s'"),
+                _("Unable to find correct value in 'bonding/mode' for '%1$s'"),
                 name);
         goto error;
     }
     if (virStrToLong_i(tmp_str + 1, NULL, 10, &tmp_int) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not parse 'bonding/mode' '%s' for '%s'"),
+                _("Could not parse 'bonding/mode' '%1$s' for '%2$s'"),
                 tmp_str, name);
         goto error;
     }
@@ -687,29 +642,29 @@ udevGetIfaceDefBond(struct udev *udev,
 
     /* bonding/arp_validate is in the format: "none 0" so we find the
      * space and increment the pointer to get the number and convert
-     * it to an interger.
+     * it to an integer.
      */
     tmp_str = udev_device_get_sysattr_value(dev, "bonding/arp_validate");
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not retrieve 'bonding/arp_validate' for '%s'"), name);
+                _("Could not retrieve 'bonding/arp_validate' for '%1$s'"), name);
         goto error;
     }
     tmp_str = strchr(tmp_str, ' ');
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Invalid format for 'bonding/arp_validate' for '%s'"), name);
+                _("Invalid format for 'bonding/arp_validate' for '%1$s'"), name);
         goto error;
     }
     if (strlen(tmp_str) < 2) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Unable to find correct value in 'bonding/arp_validate' "
-                "for '%s'"), name);
+                _("Unable to find correct value in 'bonding/arp_validate' for '%1$s'"),
+                name);
         goto error;
     }
     if (virStrToLong_i(tmp_str + 1, NULL, 10, &tmp_int) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not parse 'bonding/arp_validate' '%s' for '%s'"),
+                _("Could not parse 'bonding/arp_validate' '%1$s' for '%2$s'"),
                 tmp_str, name);
         goto error;
     }
@@ -719,12 +674,12 @@ udevGetIfaceDefBond(struct udev *udev,
     tmp_str = udev_device_get_sysattr_value(dev, "bonding/use_carrier");
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not retrieve 'bonding/use_carrier' for '%s'"), name);
+                _("Could not retrieve 'bonding/use_carrier' for '%1$s'"), name);
         goto error;
     }
     if (virStrToLong_i(tmp_str, NULL, 10, &tmp_int) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not parse 'bonding/use_carrier' '%s' for '%s'"),
+                _("Could not parse 'bonding/use_carrier' '%1$s' for '%2$s'"),
                 tmp_str, name);
         goto error;
     }
@@ -744,11 +699,10 @@ udevGetIfaceDefBond(struct udev *udev,
     tmp_str = udev_device_get_sysattr_value(dev, "bonding/arp_ip_target");
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not retrieve 'bonding/arp_ip_target' for '%s'"), name);
+                _("Could not retrieve 'bonding/arp_ip_target' for '%1$s'"), name);
         goto error;
     }
-    if (VIR_STRDUP(ifacedef->data.bond.target, tmp_str) < 0)
-        goto error;
+    ifacedef->data.bond.target = g_strdup(tmp_str);
 
     /* Slaves of the bond */
     /* Get each slave in the bond */
@@ -757,13 +711,13 @@ udevGetIfaceDefBond(struct udev *udev,
 
     if (slave_count < 0) {
         virReportSystemError(errno,
-                _("Could not get slaves of bond '%s'"), name);
+                             _("Could not get slaves of bond '%1$s'"), name);
         goto error;
     }
 
     /* Allocate our list of slave devices */
-    if (VIR_ALLOC_N(ifacedef->data.bond.itf, slave_count) < 0)
-        goto error;
+    ifacedef->data.bond.itf = g_new0(struct _virInterfaceDef *,
+                                     slave_count);
     ifacedef->data.bond.nbItf = slave_count;
 
     for (i = 0; i < slave_count; i++) {
@@ -773,8 +727,8 @@ udevGetIfaceDefBond(struct udev *udev,
         tmp_str = strchr(slave_list[i]->d_name, '_');
         if (!tmp_str || strlen(tmp_str) < 2) {
             virReportError(VIR_ERR_INTERNAL_ERROR,
-                           _("Invalid enslaved interface name '%s' seen for "
-                             "bond '%s'"), slave_list[i]->d_name, name);
+                           _("Invalid enslaved interface name '%1$s' seen for bond '%2$s'"),
+                           slave_list[i]->d_name, name);
             goto error;
         }
         /* go past the _ */
@@ -784,8 +738,8 @@ udevGetIfaceDefBond(struct udev *udev,
             udevGetIfaceDef(udev, tmp_str);
         if (!ifacedef->data.bond.itf[i]) {
             virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not get interface information for '%s', which is "
-                  "a enslaved in bond '%s'"), slave_list[i]->d_name, name);
+                           _("Could not get interface information for '%1$s', which is a enslaved in bond '%2$s'"),
+                           slave_list[i]->d_name, name);
             goto error;
         }
         VIR_FREE(slave_list[i]);
@@ -805,7 +759,7 @@ udevGetIfaceDefBond(struct udev *udev,
 
 static int
 ATTRIBUTE_NONNULL(1) ATTRIBUTE_NONNULL(2) ATTRIBUTE_NONNULL(3)
-ATTRIBUTE_NONNULL(4) ATTRIBUTE_RETURN_CHECK
+ATTRIBUTE_NONNULL(4) G_GNUC_WARN_UNUSED_RESULT
 udevGetIfaceDefBridge(struct udev *udev,
                       struct udev_device *dev,
                       const char *name,
@@ -825,25 +779,26 @@ udevGetIfaceDefBridge(struct udev *udev,
     tmp_str = udev_device_get_sysattr_value(dev, "bridge/forward_delay");
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not retrieve 'bridge/forward_delay' for '%s'"), name);
+                       _("Could not retrieve 'bridge/forward_delay' for '%1$s'"),
+                       name);
         goto error;
     }
 
-    if (VIR_STRDUP(ifacedef->data.bridge.delay, tmp_str) < 0)
-        goto error;
+    ifacedef->data.bridge.delay = g_strdup(tmp_str);
 
     /* Retrieve Spanning Tree State. Valid values = -1, 0, 1 */
     tmp_str = udev_device_get_sysattr_value(dev, "bridge/stp_state");
     if (!tmp_str) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-            _("Could not retrieve 'bridge/stp_state' for '%s'"), name);
+                       _("Could not retrieve 'bridge/stp_state' for '%1$s'"),
+                       name);
         goto error;
     }
 
     if (virStrToLong_i(tmp_str, NULL, 10, &stp) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not parse 'bridge/stp_state' '%s' for '%s'"),
-                tmp_str, name);
+                       _("Could not parse 'bridge/stp_state' '%1$s' for '%2$s'"),
+                       tmp_str, name);
         goto error;
     }
 
@@ -855,15 +810,13 @@ udevGetIfaceDefBridge(struct udev *udev,
         break;
     default:
         virReportError(VIR_ERR_INTERNAL_ERROR,
-            _("Invalid STP state value %d received for '%s'. Must be "
-              "-1, 0, or 1."), stp, name);
+                       _("Invalid STP state value %1$d received for '%2$s'. Must be -1, 0, or 1."),
+                       stp, name);
         goto error;
     }
 
     /* Members of the bridge */
-    if (virAsprintf(&member_path, "%s/%s",
-                udev_device_get_syspath(dev), "brif") < 0)
-        goto error;
+    member_path = g_strdup_printf("%s/%s", udev_device_get_syspath(dev), "brif");
 
     /* Get each member of the bridge */
     member_count = scandir(member_path, &member_list,
@@ -874,24 +827,23 @@ udevGetIfaceDefBridge(struct udev *udev,
 
     if (member_count < 0) {
         virReportSystemError(errno,
-                _("Could not get members of bridge '%s'"),
+                _("Could not get members of bridge '%1$s'"),
                 name);
         goto error;
     }
 
     /* Allocate our list of member devices */
-    if (VIR_ALLOC_N(ifacedef->data.bridge.itf, member_count) < 0)
-        goto error;
+    ifacedef->data.bridge.itf = g_new0(struct _virInterfaceDef *, member_count);
     ifacedef->data.bridge.nbItf = member_count;
 
-    /* Get the interface defintions for each member of the bridge */
+    /* Get the interface definitions for each member of the bridge */
     for (i = 0; i < member_count; i++) {
         ifacedef->data.bridge.itf[i] =
             udevGetIfaceDef(udev, member_list[i]->d_name);
         if (!ifacedef->data.bridge.itf[i]) {
             virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not get interface information for '%s', which is "
-                  "a member of bridge '%s'"), member_list[i]->d_name, name);
+                _("Could not get interface information for '%1$s', which is a member of bridge '%2$s'"),
+                member_list[i]->d_name, name);
             goto error;
         }
         VIR_FREE(member_list[i]);
@@ -911,103 +863,88 @@ udevGetIfaceDefBridge(struct udev *udev,
 
 static int
 ATTRIBUTE_NONNULL(1) ATTRIBUTE_NONNULL(2) ATTRIBUTE_NONNULL(3)
-ATTRIBUTE_NONNULL(4) ATTRIBUTE_RETURN_CHECK
-udevGetIfaceDefVlan(struct udev *udev ATTRIBUTE_UNUSED,
-                    struct udev_device *dev ATTRIBUTE_UNUSED,
+ATTRIBUTE_NONNULL(4) G_GNUC_WARN_UNUSED_RESULT
+udevGetIfaceDefVlan(struct udev *udev G_GNUC_UNUSED,
+                    struct udev_device *dev G_GNUC_UNUSED,
                     const char *name,
                     virInterfaceDef *ifacedef)
 {
-    char *procpath = NULL;
-    char *buf = NULL;
+    g_autofree char *procpath = NULL;
+    g_autofree char *buf = NULL;
     char *vid_pos, *dev_pos;
     size_t vid_len, dev_len;
     const char *vid_prefix = "VID: ";
     const char *dev_prefix = "\nDevice: ";
-    int ret = -1;
 
-    if (virAsprintf(&procpath, "/proc/net/vlan/%s", name) < 0)
-        goto cleanup;
+    procpath = g_strdup_printf("/proc/net/vlan/%s", name);
 
     if (virFileReadAll(procpath, BUFSIZ, &buf) < 0)
-        goto cleanup;
+        return -1;
 
     if ((vid_pos = strstr(buf, vid_prefix)) == NULL) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("failed to find the VID for the VLAN device '%s'"),
+                       _("failed to find the VID for the VLAN device '%1$s'"),
                        name);
-        goto cleanup;
+        return -1;
     }
     vid_pos += strlen(vid_prefix);
 
     if ((vid_len = strspn(vid_pos, "0123456789")) == 0 ||
-        !c_isspace(vid_pos[vid_len])) {
+        !g_ascii_isspace(vid_pos[vid_len])) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("failed to find the VID for the VLAN device '%s'"),
+                       _("failed to find the VID for the VLAN device '%1$s'"),
                        name);
-        goto cleanup;
+        return -1;
     }
 
     if ((dev_pos = strstr(vid_pos + vid_len, dev_prefix)) == NULL) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("failed to find the real device for the VLAN device '%s'"),
+                       _("failed to find the real device for the VLAN device '%1$s'"),
                        name);
-        goto cleanup;
+        return -1;
     }
     dev_pos += strlen(dev_prefix);
 
     if ((dev_len = strcspn(dev_pos, "\n")) == 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("failed to find the real device for the VLAN device '%s'"),
+                       _("failed to find the real device for the VLAN device '%1$s'"),
                        name);
-        goto cleanup;
+        return -1;
     }
 
-    if (VIR_STRNDUP(ifacedef->data.vlan.tag, vid_pos, vid_len) < 0)
-        goto cleanup;
-    if (VIR_STRNDUP(ifacedef->data.vlan.dev_name, dev_pos, dev_len) < 0) {
-        VIR_FREE(ifacedef->data.vlan.tag);
-        goto cleanup;
-    }
+    ifacedef->data.vlan.tag = g_strndup(vid_pos, vid_len);
+    ifacedef->data.vlan.dev_name = g_strndup(dev_pos, dev_len);
 
-    ret = 0;
-
- cleanup:
-    VIR_FREE(procpath);
-    VIR_FREE(buf);
-    return ret;
+    return 0;
 }
 
 static virInterfaceDef * ATTRIBUTE_NONNULL(1)
 udevGetIfaceDef(struct udev *udev, const char *name)
 {
     struct udev_device *dev = NULL;
-    virInterfaceDef *ifacedef;
+    g_autoptr(virInterfaceDef) ifacedef = NULL;
     unsigned int mtu;
     const char *mtu_str;
     char *vlan_parent_dev = NULL;
     const char *devtype;
 
     /* Allocate our interface definition structure */
-    if (VIR_ALLOC(ifacedef) < 0)
-        return NULL;
+    ifacedef = g_new0(virInterfaceDef, 1);
 
     /* Clear our structure and set safe defaults */
     ifacedef->startmode = VIR_INTERFACE_START_UNSPECIFIED;
-    if (VIR_STRDUP(ifacedef->name, name) < 0)
-        goto error;
+    ifacedef->name = g_strdup(name);
 
     /* Lookup the device we've been asked about */
     dev = udev_device_new_from_subsystem_sysname(udev, "net", name);
     if (!dev) {
         virReportError(VIR_ERR_NO_INTERFACE,
-                       _("couldn't find interface named '%s'"), name);
+                       _("couldn't find interface named '%1$s'"), name);
         goto error;
     }
 
     /* MAC address */
-    if (VIR_STRDUP(ifacedef->mac,
-                   udev_device_get_sysattr_value(dev, "address")) < 0)
-        goto error;
+    ifacedef->mac = g_strdup(udev_device_get_sysattr_value(dev, "address"));
 
     /* Link state and speed */
     if (virNetDevGetLinkInfo(ifacedef->name, &ifacedef->lnk) < 0)
@@ -1015,9 +952,9 @@ udevGetIfaceDef(struct udev *udev, const char *name)
 
     /* MTU */
     mtu_str = udev_device_get_sysattr_value(dev, "mtu");
-    if (virStrToLong_ui(mtu_str, NULL, 10, &mtu) < 0) {
+    if (!mtu_str || virStrToLong_ui(mtu_str, NULL, 10, &mtu) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                _("Could not parse MTU value '%s'"), mtu_str);
+                _("Could not parse MTU value '%1$s'"), NULLSTR(mtu_str));
         goto error;
     }
     ifacedef->mtu = mtu;
@@ -1078,13 +1015,11 @@ udevGetIfaceDef(struct udev *udev, const char *name)
 
     udev_device_unref(dev);
 
-    return ifacedef;
+    return g_steal_pointer(&ifacedef);
 
  error:
     udev_device_unref(dev);
 
-    virInterfaceDefFree(ifacedef);
-
     return NULL;
 }
 
@@ -1093,7 +1028,7 @@ udevInterfaceGetXMLDesc(virInterfacePtr ifinfo,
                         unsigned int flags)
 {
     struct udev *udev = udev_ref(driver->udev);
-    virInterfaceDef *ifacedef;
+    g_autoptr(virInterfaceDef) ifacedef = NULL;
     char *xmlstr = NULL;
 
     virCheckFlags(VIR_INTERFACE_XML_INACTIVE, NULL);
@@ -1111,8 +1046,6 @@ udevInterfaceGetXMLDesc(virInterfacePtr ifinfo,
 
     xmlstr = virInterfaceDefFormat(ifacedef);
 
-    virInterfaceDefFree(ifacedef);
-
  cleanup:
     /* decrement our udev ptr */
     udev_unref(udev);
@@ -1125,14 +1058,14 @@ udevInterfaceIsActive(virInterfacePtr ifinfo)
 {
     struct udev *udev = udev_ref(driver->udev);
     struct udev_device *dev;
-    virInterfaceDefPtr def = NULL;
+    g_autoptr(virInterfaceDef) def = NULL;
     int status = -1;
 
     dev = udev_device_new_from_subsystem_sysname(udev, "net",
                                                  ifinfo->name);
     if (!dev) {
         virReportError(VIR_ERR_NO_INTERFACE,
-                       _("couldn't find interface named '%s'"),
+                       _("couldn't find interface named '%1$s'"),
                        ifinfo->name);
         goto cleanup;
     }
@@ -1144,26 +1077,56 @@ udevInterfaceIsActive(virInterfacePtr ifinfo)
        goto cleanup;
 
     /* Check if it's active or not */
-    status = STREQ(udev_device_get_sysattr_value(dev, "operstate"), "up");
+    status = STREQ_NULLABLE(udev_device_get_sysattr_value(dev, "operstate"), "up");
 
     udev_device_unref(dev);
 
  cleanup:
     udev_unref(udev);
-    virInterfaceDefFree(def);
 
     return status;
 }
 
 
 static int
-udevStateInitialize(bool privileged ATTRIBUTE_UNUSED,
-                    virStateInhibitCallback callback ATTRIBUTE_UNUSED,
-                    void *opaque ATTRIBUTE_UNUSED)
-{
-    int ret = -1;
+udevStateCleanup(void);
 
-    if (VIR_ALLOC(driver) < 0)
+static virDrvStateInitResult
+udevStateInitialize(bool privileged,
+                    const char *root,
+                    bool monolithic G_GNUC_UNUSED,
+                    virStateInhibitCallback callback G_GNUC_UNUSED,
+                    void *opaque G_GNUC_UNUSED)
+{
+    int ret = VIR_DRV_STATE_INIT_ERROR;
+
+    if (root != NULL) {
+        virReportError(VIR_ERR_INVALID_ARG, "%s",
+                       _("Driver does not support embedded mode"));
+        return -1;
+    }
+
+    driver = g_new0(struct udev_iface_driver, 1);
+
+    driver->lockFD = -1;
+
+    if (privileged) {
+        driver->stateDir = g_strdup_printf("%s/libvirt/interface", RUNSTATEDIR);
+    } else {
+        g_autofree char *rundir = NULL;
+
+        rundir = virGetUserRuntimeDirectory();
+        driver->stateDir = g_strdup_printf("%s/interface/run", rundir);
+    }
+
+    if (g_mkdir_with_parents(driver->stateDir, S_IRWXU) < 0) {
+        virReportSystemError(errno, _("cannot create state directory '%1$s'"),
+                             driver->stateDir);
+        goto cleanup;
+    }
+
+    if ((driver->lockFD =
+         virPidFileAcquire(driver->stateDir, "driver", getpid())) < 0)
         goto cleanup;
 
     driver->udev = udev_new();
@@ -1172,10 +1135,13 @@ udevStateInitialize(bool privileged ATTRIBUTE_UNUSED,
                        _("failed to create udev context"));
         goto cleanup;
     }
+    driver->privileged = privileged;
 
-    ret = 0;
+    ret = VIR_DRV_STATE_INIT_COMPLETE;
 
  cleanup:
+    if (ret < 0)
+        udevStateCleanup();
     return ret;
 }
 
@@ -1185,13 +1151,69 @@ udevStateCleanup(void)
     if (!driver)
         return -1;
 
-    udev_unref(driver->udev);
+    if (driver->udev)
+        udev_unref(driver->udev);
 
+    if (driver->lockFD != -1)
+        virPidFileRelease(driver->stateDir, "driver", driver->lockFD);
+
+    VIR_FREE(driver->stateDir);
     VIR_FREE(driver);
     return 0;
 }
 
 
+static virDrvOpenStatus
+udevConnectOpen(virConnectPtr conn,
+                virConnectAuthPtr auth G_GNUC_UNUSED,
+                virConf *conf G_GNUC_UNUSED,
+                unsigned int flags)
+{
+    virCheckFlags(VIR_CONNECT_RO, VIR_DRV_OPEN_ERROR);
+
+    if (driver == NULL) {
+        virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                       _("interface state driver is not active"));
+        return VIR_DRV_OPEN_ERROR;
+    }
+
+    if (!virConnectValidateURIPath(conn->uri->path,
+                                   "interface",
+                                   driver->privileged))
+        return VIR_DRV_OPEN_ERROR;
+
+    if (virConnectOpenEnsureACL(conn) < 0)
+        return VIR_DRV_OPEN_ERROR;
+
+    return VIR_DRV_OPEN_SUCCESS;
+}
+
+static int udevConnectClose(virConnectPtr conn G_GNUC_UNUSED)
+{
+    return 0;
+}
+
+
+static int udevConnectIsSecure(virConnectPtr conn G_GNUC_UNUSED)
+{
+    /* Trivially secure, since always inside the daemon */
+    return 1;
+}
+
+
+static int udevConnectIsEncrypted(virConnectPtr conn G_GNUC_UNUSED)
+{
+    /* Not encrypted, but remote driver takes care of that */
+    return 0;
+}
+
+
+static int udevConnectIsAlive(virConnectPtr conn G_GNUC_UNUSED)
+{
+    return 1;
+}
+
+
 static virInterfaceDriver udevIfaceDriver = {
     .name = "udev",
     .connectNumOfInterfaces = udevConnectNumOfInterfaces, /* 1.0.0 */
@@ -1205,6 +1227,25 @@ static virInterfaceDriver udevIfaceDriver = {
     .interfaceGetXMLDesc = udevInterfaceGetXMLDesc, /* 1.0.0 */
 };
 
+
+static virHypervisorDriver udevHypervisorDriver = {
+    .name = "interface",
+    .connectOpen = udevConnectOpen, /* 4.1.0 */
+    .connectClose = udevConnectClose, /* 4.1.0 */
+    .connectIsEncrypted = udevConnectIsEncrypted, /* 4.1.0 */
+    .connectIsSecure = udevConnectIsSecure, /* 4.1.0 */
+    .connectIsAlive = udevConnectIsAlive, /* 4.1.0 */
+};
+
+
+static virConnectDriver udevConnectDriver = {
+    .localOnly = true,
+    .uriSchemes = (const char *[]){ "interface", NULL },
+    .hypervisorDriver = &udevHypervisorDriver,
+    .interfaceDriver = &udevIfaceDriver,
+};
+
+
 static virStateDriver interfaceStateDriver = {
     .name = "udev",
     .stateInitialize = udevStateInitialize,
@@ -1214,6 +1255,8 @@ static virStateDriver interfaceStateDriver = {
 int
 udevIfaceRegister(void)
 {
+    if (virRegisterConnectDriver(&udevConnectDriver, false) < 0)
+        return -1;
     if (virSetSharedInterfaceDriver(&udevIfaceDriver) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                        _("failed to register udev interface driver"));
