@@ -24,6 +24,7 @@
 #include "lock_driver.h"
 #include "virconf.h"
 #include "viralloc.h"
+#include "vircommand.h"
 #include "vircrypto.h"
 #include "virlog.h"
 #include "viruuid.h"
@@ -32,7 +33,8 @@
 #include "rpc/virnetclient.h"
 #include "lock_protocol.h"
 #include "configmake.h"
-#include "virstring.h"
+#include "virstoragefile.h"
+#include "virutil.h"
 
 #include "lock_driver_lockd.h"
 
@@ -41,13 +43,10 @@
 VIR_LOG_INIT("locking.lock_driver_lockd");
 
 typedef struct _virLockManagerLockDaemonPrivate virLockManagerLockDaemonPrivate;
-typedef virLockManagerLockDaemonPrivate *virLockManagerLockDaemonPrivatePtr;
 
 typedef struct _virLockManagerLockDaemonResource virLockManagerLockDaemonResource;
-typedef virLockManagerLockDaemonResource *virLockManagerLockDaemonResourcePtr;
 
 typedef struct _virLockManagerLockDaemonDriver virLockManagerLockDaemonDriver;
-typedef virLockManagerLockDaemonDriver *virLockManagerLockDaemonDriverPtr;
 
 struct _virLockManagerLockDaemonResource {
     char *lockspace;
@@ -62,7 +61,7 @@ struct _virLockManagerLockDaemonPrivate {
     pid_t pid;
 
     size_t nresources;
-    virLockManagerLockDaemonResourcePtr resources;
+    virLockManagerLockDaemonResource *resources;
 
     bool hasRWDisks;
 };
@@ -77,17 +76,16 @@ struct _virLockManagerLockDaemonDriver {
     char *scsiLockSpaceDir;
 };
 
-static virLockManagerLockDaemonDriverPtr driver;
+static virLockManagerLockDaemonDriver *driver;
 
 static int virLockManagerLockDaemonLoadConfig(const char *configFile)
 {
-    virConfPtr conf;
-    virConfValuePtr p;
+    g_autoptr(virConf) conf = NULL;
 
     if (access(configFile, R_OK) == -1) {
         if (errno != ENOENT) {
             virReportSystemError(errno,
-                                 _("Unable to access config file %s"),
+                                 _("Unable to access config file %1$s"),
                                  configFile);
             return -1;
         }
@@ -97,56 +95,22 @@ static int virLockManagerLockDaemonLoadConfig(const char *configFile)
     if (!(conf = virConfReadFile(configFile, 0)))
         return -1;
 
-#define CHECK_TYPE(name, typ) if (p && p->type != (typ)) {              \
-        virReportError(VIR_ERR_INTERNAL_ERROR,                          \
-                       "%s: %s: expected type " #typ,                   \
-                       configFile, (name));                             \
-        virConfFree(conf);                                              \
-        return -1;                                                      \
-    }
+    if (virConfGetValueBool(conf, "auto_disk_leases", &driver->autoDiskLease) < 0)
+        return -1;
 
-    p = virConfGetValue(conf, "auto_disk_leases");
-    CHECK_TYPE("auto_disk_leases", VIR_CONF_ULONG);
-    if (p) driver->autoDiskLease = p->l;
+    if (virConfGetValueString(conf, "file_lockspace_dir", &driver->fileLockSpaceDir) < 0)
+        return -1;
 
-    p = virConfGetValue(conf, "file_lockspace_dir");
-    CHECK_TYPE("file_lockspace_dir", VIR_CONF_STRING);
-    if (p && p->str) {
-        VIR_FREE(driver->fileLockSpaceDir);
-        if (VIR_STRDUP(driver->fileLockSpaceDir, p->str) < 0) {
-            virConfFree(conf);
-            return -1;
-        }
-    }
+    if (virConfGetValueString(conf, "lvm_lockspace_dir", &driver->lvmLockSpaceDir) < 0)
+        return -1;
 
-    p = virConfGetValue(conf, "lvm_lockspace_dir");
-    CHECK_TYPE("lvm_lockspace_dir", VIR_CONF_STRING);
-    if (p && p->str) {
-        VIR_FREE(driver->lvmLockSpaceDir);
-        if (VIR_STRDUP(driver->lvmLockSpaceDir, p->str) < 0) {
-            virConfFree(conf);
-            return -1;
-        }
-    }
+    if (virConfGetValueString(conf, "scsi_lockspace_dir", &driver->scsiLockSpaceDir) < 0)
+        return -1;
 
-    p = virConfGetValue(conf, "scsi_lockspace_dir");
-    CHECK_TYPE("scsi_lockspace_dir", VIR_CONF_STRING);
-    if (p && p->str) {
-        VIR_FREE(driver->scsiLockSpaceDir);
-        if (VIR_STRDUP(driver->scsiLockSpaceDir, p->str) < 0) {
-            virConfFree(conf);
-            return -1;
-        }
-    }
+    driver->requireLeaseForDisks = !driver->autoDiskLease;
+    if (virConfGetValueBool(conf, "require_lease_for_disks", &driver->requireLeaseForDisks) < 0)
+        return -1;
 
-    p = virConfGetValue(conf, "require_lease_for_disks");
-    CHECK_TYPE("require_lease_for_disks", VIR_CONF_ULONG);
-    if (p)
-        driver->requireLeaseForDisks = p->l;
-    else
-        driver->requireLeaseForDisks = !driver->autoDiskLease;
-
-    virConfFree(conf);
     return 0;
 }
 
@@ -155,36 +119,26 @@ static char *virLockManagerLockDaemonPath(bool privileged)
 {
     char *path;
     if (privileged) {
-        if (VIR_STRDUP(path, LOCALSTATEDIR "/run/libvirt/virtlockd-sock") < 0)
-            return NULL;
+        path = g_strdup(RUNSTATEDIR "/libvirt/virtlockd-sock");
     } else {
-        char *rundir = NULL;
+        g_autofree char *rundir = NULL;
 
-        if (!(rundir = virGetUserRuntimeDirectory()))
-            return NULL;
+        rundir = virGetUserRuntimeDirectory();
 
-        if (virAsprintf(&path, "%s/virtlockd-sock", rundir) < 0) {
-            VIR_FREE(rundir);
-            return NULL;
-        }
-
-        VIR_FREE(rundir);
+        path = g_strdup_printf("%s/virtlockd-sock", rundir);
     }
     return path;
 }
 
 
 static int
-virLockManagerLockDaemonConnectionRegister(virLockManagerPtr lock,
-                                           virNetClientPtr client,
-                                           virNetClientProgramPtr program,
+virLockManagerLockDaemonConnectionRegister(virLockManager *lock,
+                                           virNetClient *client,
+                                           virNetClientProgram *program,
                                            int *counter)
 {
-    virLockManagerLockDaemonPrivatePtr priv = lock->privateData;
-    virLockSpaceProtocolRegisterArgs args;
-    int rv = -1;
-
-    memset(&args, 0, sizeof(args));
+    virLockManagerLockDaemonPrivate *priv = lock->privateData;
+    virLockSpaceProtocolRegisterArgs args = { 0 };
 
     args.flags = 0;
     memcpy(args.owner.uuid, priv->uuid, VIR_UUID_BUFLEN);
@@ -199,25 +153,19 @@ virLockManagerLockDaemonConnectionRegister(virLockManagerPtr lock,
                                 0, NULL, NULL, NULL,
                                 (xdrproc_t)xdr_virLockSpaceProtocolRegisterArgs, (char*)&args,
                                 (xdrproc_t)xdr_void, NULL) < 0)
-        goto cleanup;
+        return -1;
 
-    rv = 0;
-
- cleanup:
-    return rv;
+    return 0;
 }
 
 
 static int
-virLockManagerLockDaemonConnectionRestrict(virLockManagerPtr lock ATTRIBUTE_UNUSED,
-                                           virNetClientPtr client,
-                                           virNetClientProgramPtr program,
+virLockManagerLockDaemonConnectionRestrict(virLockManager *lock G_GNUC_UNUSED,
+                                           virNetClient *client,
+                                           virNetClientProgram *program,
                                            int *counter)
 {
-    virLockSpaceProtocolRestrictArgs args;
-    int rv = -1;
-
-    memset(&args, 0, sizeof(args));
+    virLockSpaceProtocolRestrictArgs args = { 0 };
 
     args.flags = 0;
 
@@ -228,21 +176,18 @@ virLockManagerLockDaemonConnectionRestrict(virLockManagerPtr lock ATTRIBUTE_UNUS
                                 0, NULL, NULL, NULL,
                                 (xdrproc_t)xdr_virLockSpaceProtocolRestrictArgs, (char*)&args,
                                 (xdrproc_t)xdr_void, NULL) < 0)
-        goto cleanup;
+        return -1;
 
-    rv = 0;
-
- cleanup:
-    return rv;
+    return 0;
 }
 
 
-static virNetClientPtr virLockManagerLockDaemonConnectionNew(bool privileged,
-                                                             virNetClientProgramPtr *prog)
+static virNetClient *virLockManagerLockDaemonConnectionNew(bool privileged,
+                                                             virNetClientProgram **prog)
 {
-    virNetClientPtr client = NULL;
-    char *lockdpath;
-    char *daemonPath = NULL;
+    virNetClient *client = NULL;
+    g_autofree char *lockdpath = NULL;
+    g_autofree char *daemonPath = NULL;
 
     *prog = NULL;
 
@@ -252,13 +197,12 @@ static virNetClientPtr virLockManagerLockDaemonConnectionNew(bool privileged,
     if (!privileged &&
         !(daemonPath = virFileFindResourceFull("virtlockd",
                                                NULL, NULL,
-                                               abs_topbuilddir "/src",
+                                               abs_top_builddir "/src",
                                                SBINDIR,
                                                "VIRTLOCKD_PATH")))
         goto error;
 
     if (!(client = virNetClientNewUNIX(lockdpath,
-                                       daemonPath != NULL,
                                        daemonPath)))
         goto error;
 
@@ -272,14 +216,9 @@ static virNetClientPtr virLockManagerLockDaemonConnectionNew(bool privileged,
     if (virNetClientAddProgram(client, *prog) < 0)
         goto error;
 
-    VIR_FREE(daemonPath);
-    VIR_FREE(lockdpath);
-
     return client;
 
  error:
-    VIR_FREE(daemonPath);
-    VIR_FREE(lockdpath);
     virNetClientClose(client);
     virObjectUnref(client);
     virObjectUnref(*prog);
@@ -287,12 +226,12 @@ static virNetClientPtr virLockManagerLockDaemonConnectionNew(bool privileged,
 }
 
 
-static virNetClientPtr
-virLockManagerLockDaemonConnect(virLockManagerPtr lock,
-                                virNetClientProgramPtr *program,
+static virNetClient *
+virLockManagerLockDaemonConnect(virLockManager *lock,
+                                virNetClientProgram **program,
                                 int *counter)
 {
-    virNetClientPtr client;
+    virNetClient *client;
 
     if (!(client = virLockManagerLockDaemonConnectionNew(geteuid() == 0, program)))
         return NULL;
@@ -314,13 +253,12 @@ virLockManagerLockDaemonConnect(virLockManagerPtr lock,
 
 static int virLockManagerLockDaemonSetupLockspace(const char *path)
 {
-    virNetClientPtr client;
-    virNetClientProgramPtr program = NULL;
-    virLockSpaceProtocolCreateLockSpaceArgs args;
+    virNetClient *client;
+    virNetClientProgram *program = NULL;
+    virLockSpaceProtocolCreateLockSpaceArgs args = { 0 };
     int rv = -1;
     int counter = 0;
 
-    memset(&args, 0, sizeof(args));
     args.path = (char*)path;
 
     if (!(client = virLockManagerLockDaemonConnectionNew(geteuid() == 0, &program)))
@@ -332,16 +270,8 @@ static int virLockManagerLockDaemonSetupLockspace(const char *path)
                                 VIR_LOCK_SPACE_PROTOCOL_PROC_CREATE_LOCKSPACE,
                                 0, NULL, NULL, NULL,
                                 (xdrproc_t)xdr_virLockSpaceProtocolCreateLockSpaceArgs, (char*)&args,
-                                (xdrproc_t)xdr_void, NULL) < 0) {
-        virErrorPtr err = virGetLastError();
-        if (err && err->code == VIR_ERR_OPERATION_INVALID) {
-            /* The lockspace already exists */
-            virResetLastError();
-            rv = 0;
-        } else {
-            goto cleanup;
-        }
-    }
+                                (xdrproc_t)xdr_void, NULL) < 0)
+        goto cleanup;
 
     rv = 0;
 
@@ -359,15 +289,14 @@ static int virLockManagerLockDaemonInit(unsigned int version,
                                         const char *configFile,
                                         unsigned int flags)
 {
-    VIR_DEBUG("version=%u configFile=%s flags=%x", version, NULLSTR(configFile), flags);
+    VIR_DEBUG("version=%u configFile=%s flags=0x%x", version, NULLSTR(configFile), flags);
 
     virCheckFlags(0, -1);
 
     if (driver)
         return 0;
 
-    if (VIR_ALLOC(driver) < 0)
-        return -1;
+    driver = g_new0(virLockManagerLockDaemonDriver, 1);
 
     driver->requireLeaseForDisks = true;
     driver->autoDiskLease = true;
@@ -401,48 +330,55 @@ static int virLockManagerLockDaemonDeinit(void)
     if (!driver)
         return 0;
 
+    VIR_FREE(driver->scsiLockSpaceDir);
+    VIR_FREE(driver->lvmLockSpaceDir);
     VIR_FREE(driver->fileLockSpaceDir);
     VIR_FREE(driver);
 
     return 0;
 }
 
-static void virLockManagerLockDaemonFree(virLockManagerPtr lock)
+static void
+virLockManagerLockDaemonPrivateFree(virLockManagerLockDaemonPrivate *priv)
 {
-    virLockManagerLockDaemonPrivatePtr priv = lock->privateData;
     size_t i;
 
     if (!priv)
         return;
 
-    lock->privateData = NULL;
-
     for (i = 0; i < priv->nresources; i++) {
-        VIR_FREE(priv->resources[i].lockspace);
-        VIR_FREE(priv->resources[i].name);
+        g_free(priv->resources[i].lockspace);
+        g_free(priv->resources[i].name);
     }
-    VIR_FREE(priv->resources);
+    g_free(priv->resources);
 
-    VIR_FREE(priv->name);
+    g_free(priv->name);
+    g_free(priv);
+}
 
-    VIR_FREE(priv);
+static void virLockManagerLockDaemonFree(virLockManager *lock)
+{
+    if (!lock)
+        return;
+
+    g_clear_pointer(&lock->privateData, virLockManagerLockDaemonPrivateFree);
 }
 
 
-static int virLockManagerLockDaemonNew(virLockManagerPtr lock,
+static int virLockManagerLockDaemonNew(virLockManager *lock,
                                        unsigned int type,
                                        size_t nparams,
-                                       virLockManagerParamPtr params,
+                                       virLockManagerParam *params,
                                        unsigned int flags)
 {
-    virLockManagerLockDaemonPrivatePtr priv;
+    virLockManagerLockDaemonPrivate *priv = NULL;
     size_t i;
+    int ret = -1;
 
     virCheckFlags(VIR_LOCK_MANAGER_NEW_STARTED, -1);
 
-    if (VIR_ALLOC(priv) < 0)
-        return -1;
-    lock->privateData = priv;
+    priv = g_new0(virLockManagerLockDaemonPrivate,
+                  1);
 
     switch (type) {
     case VIR_LOCK_MANAGER_OBJECT_TYPE_DOMAIN:
@@ -450,8 +386,7 @@ static int virLockManagerLockDaemonNew(virLockManagerPtr lock,
             if (STREQ(params[i].key, "uuid")) {
                 memcpy(priv->uuid, params[i].value.uuid, VIR_UUID_BUFLEN);
             } else if (STREQ(params[i].key, "name")) {
-                if (VIR_STRDUP(priv->name, params[i].value.str) < 0)
-                    return -1;
+                priv->name = g_strdup(params[i].value.str);
             } else if (STREQ(params[i].key, "id")) {
                 priv->id = params[i].value.iv;
             } else if (STREQ(params[i].key, "pid")) {
@@ -460,50 +395,120 @@ static int virLockManagerLockDaemonNew(virLockManagerPtr lock,
                 /* ignored */
             } else {
                 virReportError(VIR_ERR_INTERNAL_ERROR,
-                               _("Unexpected parameter %s for object"),
+                               _("Unexpected parameter %1$s for object"),
                                params[i].key);
+                goto cleanup;
             }
         }
         if (priv->id == 0) {
             virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                            _("Missing ID parameter for domain object"));
-            return -1;
+            goto cleanup;
         }
         if (priv->pid == 0)
             VIR_DEBUG("Missing PID parameter for domain object");
         if (!priv->name) {
             virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                            _("Missing name parameter for domain object"));
-            return -1;
+            goto cleanup;
         }
         if (!virUUIDIsValid(priv->uuid)) {
             virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                            _("Missing UUID parameter for domain object"));
-            return -1;
+            goto cleanup;
         }
         break;
 
     default:
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("Unknown lock manager object type %d"),
+                       _("Unknown lock manager object type %1$d"),
                        type);
-        return -1;
+        goto cleanup;
     }
 
-    return 0;
+    lock->privateData = g_steal_pointer(&priv);
+    ret = 0;
+ cleanup:
+    virLockManagerLockDaemonPrivateFree(priv);
+    return ret;
 }
 
 
-static int virLockManagerLockDaemonAddResource(virLockManagerPtr lock,
+#ifdef LVS
+static int
+virLockManagerGetLVMKey(const char *path,
+                        char **key)
+{
+    /*
+     *  # lvs --noheadings --unbuffered --nosuffix --options "uuid" LVNAME
+     *    06UgP5-2rhb-w3Bo-3mdR-WeoL-pytO-SAa2ky
+     */
+    int status;
+    int ret = -1;
+    g_autoptr(virCommand) cmd = NULL;
+
+    cmd = virCommandNewArgList(LVS, "--noheadings",
+                               "--unbuffered", "--nosuffix",
+                               "--options", "uuid", path,
+                               NULL
+                               );
+    *key = NULL;
+
+    /* Run the program and capture its output */
+    virCommandSetOutputBuffer(cmd, key);
+    if (virCommandRun(cmd, &status) < 0)
+        goto cleanup;
+
+    /* Explicitly check status == 0, rather than passing NULL
+     * to virCommandRun because we don't want to raise an actual
+     * error in this scenario, just return a NULL key.
+     */
+
+    if (status == 0 && *key) {
+        char *nl;
+        char *tmp = *key;
+
+        /* Find first non-space character */
+        while (*tmp && g_ascii_isspace(*tmp))
+            tmp++;
+        /* Kill leading spaces */
+        if (tmp != *key)
+            memmove(*key, tmp, strlen(tmp)+1);
+
+        /* Kill trailing newline */
+        if ((nl = strchr(*key, '\n')))
+            *nl = '\0';
+    }
+
+    ret = 0;
+
+ cleanup:
+    if (*key && STREQ(*key, ""))
+        VIR_FREE(*key);
+
+    return ret;
+}
+#else
+static int
+virLockManagerGetLVMKey(const char *path,
+                        char **key G_GNUC_UNUSED)
+{
+    virReportSystemError(ENOSYS, _("Unable to get LVM key for %1$s"), path);
+    return -1;
+}
+#endif
+
+
+static int virLockManagerLockDaemonAddResource(virLockManager *lock,
                                                unsigned int type,
                                                const char *name,
                                                size_t nparams,
-                                               virLockManagerParamPtr params,
+                                               virLockManagerParam *params,
                                                unsigned int flags)
 {
-    virLockManagerLockDaemonPrivatePtr priv = lock->privateData;
-    char *newName = NULL;
-    char *newLockspace = NULL;
+    virLockManagerLockDaemonPrivate *priv = lock->privateData;
+    g_autofree char *newName = NULL;
+    g_autofree char *newLockspace = NULL;
     bool autoCreate = false;
 
     virCheckFlags(VIR_LOCK_MANAGER_RESOURCE_READONLY |
@@ -532,13 +537,12 @@ static int virLockManagerLockDaemonAddResource(virLockManagerPtr lock,
         if (STRPREFIX(name, "/dev") &&
             driver->lvmLockSpaceDir) {
             VIR_DEBUG("Trying to find an LVM UUID for %s", name);
-            if (virStorageFileGetLVMKey(name, &newName) < 0)
-                goto error;
+            if (virLockManagerGetLVMKey(name, &newName) < 0)
+                return -1;
 
             if (newName) {
                 VIR_DEBUG("Got an LVM UUID %s for %s", newName, name);
-                if (VIR_STRDUP(newLockspace, driver->lvmLockSpaceDir) < 0)
-                    goto error;
+                newLockspace = g_strdup(driver->lvmLockSpaceDir);
                 autoCreate = true;
                 break;
             }
@@ -549,13 +553,12 @@ static int virLockManagerLockDaemonAddResource(virLockManagerPtr lock,
         if (STRPREFIX(name, "/dev") &&
             driver->scsiLockSpaceDir) {
             VIR_DEBUG("Trying to find an SCSI ID for %s", name);
-            if (virStorageFileGetSCSIKey(name, &newName) < 0)
-                goto error;
+            if (virStorageFileGetSCSIKey(name, &newName, false) < 0)
+                return -1;
 
             if (newName) {
                 VIR_DEBUG("Got an SCSI ID %s for %s", newName, name);
-                if (VIR_STRDUP(newLockspace, driver->scsiLockSpaceDir) < 0)
-                    goto error;
+                newLockspace = g_strdup(driver->scsiLockSpaceDir);
                 autoCreate = true;
                 break;
             }
@@ -564,17 +567,14 @@ static int virLockManagerLockDaemonAddResource(virLockManagerPtr lock,
         }
 
         if (driver->fileLockSpaceDir) {
-            if (VIR_STRDUP(newLockspace, driver->fileLockSpaceDir) < 0)
-                goto error;
+            newLockspace = g_strdup(driver->fileLockSpaceDir);
             if (virCryptoHashString(VIR_CRYPTO_HASH_SHA256, name, &newName) < 0)
-                goto error;
+                return -1;
             autoCreate = true;
             VIR_DEBUG("Using indirect lease %s for %s", newName, name);
         } else {
-            if (VIR_STRDUP(newLockspace, "") < 0)
-                goto error;
-            if (VIR_STRDUP(newName, name) < 0)
-                goto error;
+            newLockspace = g_strdup("");
+            newName = g_strdup(name);
             VIR_DEBUG("Using direct lease for %s", name);
         }
 
@@ -596,7 +596,7 @@ static int virLockManagerLockDaemonAddResource(virLockManagerPtr lock,
                 path = params[i].value.str;
             } else {
                 virReportError(VIR_ERR_INTERNAL_ERROR,
-                               _("Unexpected parameter %s for lease resource"),
+                               _("Unexpected parameter %1$s for lease resource"),
                                params[i].key);
                 return -1;
             }
@@ -606,25 +606,20 @@ static int virLockManagerLockDaemonAddResource(virLockManagerPtr lock,
                            _("Missing path or lockspace for lease resource"));
             return -1;
         }
-        if (virAsprintf(&newLockspace, "%s/%s",
-                        path, lockspace) < 0)
-            return -1;
-        if (VIR_STRDUP(newName, name) < 0)
-            goto error;
+        newLockspace = g_strdup_printf("%s/%s", path, lockspace);
+        newName = g_strdup(name);
 
     }   break;
     default:
         virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("Unknown lock manager object type %d"),
+                       _("Unknown lock manager object type %1$d"),
                        type);
         return -1;
     }
 
-    if (VIR_EXPAND_N(priv->resources, priv->nresources, 1) < 0)
-        goto error;
-
-    priv->resources[priv->nresources-1].lockspace = newLockspace;
-    priv->resources[priv->nresources-1].name = newName;
+    VIR_EXPAND_N(priv->resources, priv->nresources, 1);
+    priv->resources[priv->nresources-1].lockspace = g_steal_pointer(&newLockspace);
+    priv->resources[priv->nresources-1].name = g_steal_pointer(&newName);
 
     if (flags & VIR_LOCK_MANAGER_RESOURCE_SHARED)
         priv->resources[priv->nresources-1].flags |=
@@ -635,25 +630,20 @@ static int virLockManagerLockDaemonAddResource(virLockManagerPtr lock,
             VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_AUTOCREATE;
 
     return 0;
-
- error:
-    VIR_FREE(newLockspace);
-    VIR_FREE(newName);
-    return -1;
 }
 
 
-static int virLockManagerLockDaemonAcquire(virLockManagerPtr lock,
-                                           const char *state ATTRIBUTE_UNUSED,
+static int virLockManagerLockDaemonAcquire(virLockManager *lock,
+                                           const char *state G_GNUC_UNUSED,
                                            unsigned int flags,
-                                           virDomainLockFailureAction action ATTRIBUTE_UNUSED,
+                                           virDomainLockFailureAction action G_GNUC_UNUSED,
                                            int *fd)
 {
-    virNetClientPtr client = NULL;
-    virNetClientProgramPtr program = NULL;
+    virNetClient *client = NULL;
+    virNetClientProgram *program = NULL;
     int counter = 0;
     int rv = -1;
-    virLockManagerLockDaemonPrivatePtr priv = lock->privateData;
+    virLockManagerLockDaemonPrivate *priv = lock->privateData;
 
     virCheckFlags(VIR_LOCK_MANAGER_ACQUIRE_REGISTER_ONLY |
                   VIR_LOCK_MANAGER_ACQUIRE_RESTRICT, -1);
@@ -676,12 +666,9 @@ static int virLockManagerLockDaemonAcquire(virLockManagerPtr lock,
     if (!(flags & VIR_LOCK_MANAGER_ACQUIRE_REGISTER_ONLY)) {
         size_t i;
         for (i = 0; i < priv->nresources; i++) {
-            virLockSpaceProtocolAcquireResourceArgs args;
+            virLockSpaceProtocolAcquireResourceArgs args = { 0 };
 
-            memset(&args, 0, sizeof(args));
-
-            if (priv->resources[i].lockspace)
-                args.path = priv->resources[i].lockspace;
+            args.path = priv->resources[i].lockspace;
             args.name = priv->resources[i].name;
             args.flags = priv->resources[i].flags;
 
@@ -712,16 +699,16 @@ static int virLockManagerLockDaemonAcquire(virLockManagerPtr lock,
     return rv;
 }
 
-static int virLockManagerLockDaemonRelease(virLockManagerPtr lock,
+static int virLockManagerLockDaemonRelease(virLockManager *lock,
                                            char **state,
                                            unsigned int flags)
 {
-    virNetClientPtr client = NULL;
-    virNetClientProgramPtr program = NULL;
+    virNetClient *client = NULL;
+    virNetClientProgram *program = NULL;
     int counter = 0;
     int rv = -1;
     size_t i;
-    virLockManagerLockDaemonPrivatePtr priv = lock->privateData;
+    virLockManagerLockDaemonPrivate *priv = lock->privateData;
 
     virCheckFlags(0, -1);
 
@@ -732,9 +719,7 @@ static int virLockManagerLockDaemonRelease(virLockManagerPtr lock,
         goto cleanup;
 
     for (i = 0; i < priv->nresources; i++) {
-        virLockSpaceProtocolReleaseResourceArgs args;
-
-        memset(&args, 0, sizeof(args));
+        virLockSpaceProtocolReleaseResourceArgs args = { 0 };
 
         if (priv->resources[i].lockspace)
             args.path = priv->resources[i].lockspace;
@@ -766,7 +751,7 @@ static int virLockManagerLockDaemonRelease(virLockManagerPtr lock,
 }
 
 
-static int virLockManagerLockDaemonInquire(virLockManagerPtr lock ATTRIBUTE_UNUSED,
+static int virLockManagerLockDaemonInquire(virLockManager *lock G_GNUC_UNUSED,
                                            char **state,
                                            unsigned int flags)
 {
