@@ -20,9 +20,6 @@
 
 #include <config.h>
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <time.h>
 
 #include <selinux/selinux.h>
@@ -30,10 +27,7 @@
 
 #include "internal.h"
 #include "testutils.h"
-#include "viralloc.h"
 #include "virlog.h"
-#include "virerror.h"
-#include "virfile.h"
 #include "security/security_manager.h"
 #include "virstring.h"
 
@@ -42,7 +36,7 @@
 VIR_LOG_INIT("tests.securityselinuxtest");
 
 struct testSELinuxGenLabelData {
-    virSecurityManagerPtr mgr;
+    virSecurityManager *mgr;
 
     const char *pidcon;
 
@@ -62,39 +56,37 @@ struct testSELinuxGenLabelData {
     int catMax;
 };
 
-static virDomainDefPtr
+static virDomainDef *
 testBuildDomainDef(bool dynamic,
                    const char *label,
                    const char *baselabel)
 {
-    virDomainDefPtr def;
-    virSecurityLabelDefPtr secdef;
+    g_autoptr(virDomainDef) def = NULL;
+    virSecurityLabelDef *secdef = NULL;
 
-    if (!(def = virDomainDefNew()))
-        goto error;
-
-    if (VIR_ALLOC_N(def->seclabels, 1) < 0)
-        goto error;
-
-    if (VIR_ALLOC(secdef) < 0)
+    if (!(def = virDomainDefNew(NULL)))
         goto error;
 
     def->virtType = VIR_DOMAIN_VIRT_KVM;
+    def->seclabels = g_new0(virSecurityLabelDef *, 1);
+
+    secdef = g_new0(virSecurityLabelDef, 1);
+
+    secdef->model = g_strdup("selinux");
+
+    secdef->type = dynamic ? VIR_DOMAIN_SECLABEL_DYNAMIC : VIR_DOMAIN_SECLABEL_STATIC;
+    if (label)
+        secdef->label = g_strdup(label);
+
+    if (baselabel)
+        secdef->baselabel = g_strdup(baselabel);
+
     def->seclabels[0] = secdef;
-    def->seclabels[0]->type = dynamic ? VIR_DOMAIN_SECLABEL_DYNAMIC : VIR_DOMAIN_SECLABEL_STATIC;
-
-    if (label &&
-        VIR_STRDUP(def->seclabels[0]->label, label) < 0)
-        goto error;
-
-    if (baselabel &&
-        VIR_STRDUP(def->seclabels[0]->baselabel, baselabel) < 0)
-        goto error;
-
-    return def;
+    def->nseclabels++;
+    return g_steal_pointer(&def);
 
  error:
-    virDomainDefFree(def);
+    virSecurityLabelDefFree(secdef);
     return NULL;
 }
 
@@ -105,7 +97,7 @@ testSELinuxCheckCon(context_t con,
                     const char *role,
                     const char *type,
                     int sensMin,
-                    int sensMax ATTRIBUTE_UNUSED,
+                    int sensMax G_GNUC_UNUSED,
                     int catMin,
                     int catMax)
 {
@@ -215,11 +207,11 @@ testSELinuxGenLabel(const void *opaque)
 {
     const struct testSELinuxGenLabelData *data = opaque;
     int ret = -1;
-    virDomainDefPtr def;
+    g_autoptr(virDomainDef) def = NULL;
     context_t con = NULL;
     context_t imgcon = NULL;
 
-    if (setcon_raw((security_context_t)data->pidcon) < 0) {
+    if (setcon_raw(data->pidcon) < 0) {
         perror("Cannot set process security context");
         return -1;
     }
@@ -259,7 +251,6 @@ testSELinuxGenLabel(const void *opaque)
  cleanup:
     context_free(con);
     context_free(imgcon);
-    virDomainDefFree(def);
     return ret;
 }
 
@@ -269,7 +260,7 @@ static int
 mymain(void)
 {
     int ret = 0;
-    virSecurityManagerPtr mgr;
+    virSecurityManager *mgr;
 
     if (!(mgr = virSecurityManagerNew("selinux", "QEMU",
                                       VIR_SECURITY_MANAGER_DEFAULT_CONFINED |
@@ -279,19 +270,19 @@ mymain(void)
         return EXIT_FAILURE;
     }
 
-#define DO_TEST_GEN_LABEL(desc, pidcon,                                 \
-                          dynamic, label, baselabel,                    \
-                          user, role, imageRole,                        \
-                          type, imageType,                              \
-                          sensMin, sensMax, catMin, catMax)             \
-    do {                                                                \
-        struct testSELinuxGenLabelData data = {                         \
-            mgr, pidcon, dynamic, label, baselabel,                     \
-            user, role, imageRole, type, imageType,                     \
-            sensMin, sensMax, catMin, catMax                            \
-        };                                                              \
-        if (virtTestRun("GenLabel " # desc, testSELinuxGenLabel, &data) < 0) \
-            ret = -1;                                                   \
+#define DO_TEST_GEN_LABEL(desc, pidcon, \
+                          dynamic, label, baselabel, \
+                          user, role, imageRole, \
+                          type, imageType, \
+                          sensMin, sensMax, catMin, catMax) \
+    do { \
+        struct testSELinuxGenLabelData data = { \
+            mgr, pidcon, dynamic, label, baselabel, \
+            user, role, imageRole, type, imageType, \
+            sensMin, sensMax, catMin, catMax \
+        }; \
+        if (virTestRun("GenLabel " # desc, testSELinuxGenLabel, &data) < 0) \
+            ret = -1; \
     } while (0)
 
     DO_TEST_GEN_LABEL("dynamic unconfined, s0, c0.c1023",
@@ -337,7 +328,8 @@ mymain(void)
                       "svirt_t", "svirt_image_t",
                       0, 0, 0, 1023);
 
+    virObjectUnref(mgr);
     return (ret == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
-VIRT_TEST_MAIN_PRELOAD(mymain, abs_builddir "/.libs/libsecurityselinuxhelper.so")
+VIR_TEST_MAIN_PRELOAD(mymain, "libsecurityselinuxhelper.so")
