@@ -20,63 +20,84 @@
  */
 
 #include <config.h>
-#include <unistd.h>
 
 #include "virt-host-validate-qemu.h"
 #include "virt-host-validate-common.h"
 #include "virarch.h"
 #include "virbitmap.h"
+#include "vircgroup.h"
 
 int virHostValidateQEMU(void)
 {
-    virBitmapPtr flags;
+    g_autoptr(virBitmap) flags = NULL;
     int ret = 0;
     bool hasHwVirt = false;
-
-    virHostMsgCheck("QEMU", "%s", _("for hardware virtualization"));
+    bool hasVirtFlag = false;
+    virArch arch = virArchFromHost();
+    const char *kvmhint = _("Check that CPU and firmware supports virtualization and kvm module is loaded");
 
     if (!(flags = virHostValidateGetCPUFlags()))
         return -1;
 
-    switch (virArchFromHost()) {
+    switch ((int)arch) {
     case VIR_ARCH_I686:
     case VIR_ARCH_X86_64:
+        hasVirtFlag = true;
+        kvmhint = _("Check that the 'kvm-intel' or 'kvm-amd' modules are loaded & the BIOS has enabled virtualization");
         if (virBitmapIsBitSet(flags, VIR_HOST_VALIDATE_CPU_FLAG_SVM) ||
             virBitmapIsBitSet(flags, VIR_HOST_VALIDATE_CPU_FLAG_VMX))
             hasHwVirt = true;
         break;
     case VIR_ARCH_S390:
     case VIR_ARCH_S390X:
+        hasVirtFlag = true;
         if (virBitmapIsBitSet(flags, VIR_HOST_VALIDATE_CPU_FLAG_SIE))
             hasHwVirt = true;
         break;
+    case VIR_ARCH_PPC64:
+    case VIR_ARCH_PPC64LE:
+        hasVirtFlag = true;
+        hasHwVirt = true;
+        break;
     default:
         hasHwVirt = false;
     }
 
-    if (hasHwVirt) {
-        virHostMsgPass();
+    if (hasVirtFlag) {
+        virHostMsgCheck("QEMU", "%s", _("Checking for hardware virtualization"));
+        if (hasHwVirt) {
+            virHostMsgPass();
+        } else {
+            virHostMsgFail(VIR_HOST_VALIDATE_FAIL,
+                           _("Host not compatible with KVM; HW virtualization CPU features not found. Only emulated CPUs are available; performance will be significantly limited"));
+            ret = -1;
+        }
+    }
+
+    if (hasHwVirt || !hasVirtFlag) {
         if (virHostValidateDeviceExists("QEMU", "/dev/kvm",
                                         VIR_HOST_VALIDATE_FAIL,
-                                        _("Check that the 'kvm-intel' or 'kvm-amd' modules are "
-                                          "loaded & the BIOS has enabled virtualization")) < 0)
+                                        kvmhint) <0)
             ret = -1;
         else if (virHostValidateDeviceAccessible("QEMU", "/dev/kvm",
                                                  VIR_HOST_VALIDATE_FAIL,
-                                                 _("Check /dev/kvm is world writable or you are in "
-                                                   "a group that is allowed to access it")) < 0)
+                                                 _("Check /dev/kvm is world writable or you are in a group that is allowed to access it")) < 0)
             ret = -1;
-    } else {
-        virHostMsgFail(VIR_HOST_VALIDATE_WARN,
-                       _("Only emulated CPUs are available, performance will be significantly limited"));
     }
 
-    virBitmapFree(flags);
+    if (arch == VIR_ARCH_PPC64 || arch == VIR_ARCH_PPC64LE) {
+        virHostMsgCheck("QEMU", "%s", _("Checking for PowerPC KVM module loaded"));
+
+        if (!virHostKernelModuleIsLoaded("kvm_hv"))
+            virHostMsgFail(VIR_HOST_VALIDATE_WARN,
+                          _("Load kvm_hv for better performance"));
+        else
+            virHostMsgPass();
+    }
 
     if (virHostValidateDeviceExists("QEMU", "/dev/vhost-net",
                                     VIR_HOST_VALIDATE_WARN,
-                                    _("Load the 'vhost_net' module to improve performance "
-                                      "of virtio networking")) < 0)
+                                    _("Load the 'vhost_net' module to improve performance of virtio networking")) < 0)
         ret = -1;
 
     if (virHostValidateDeviceExists("QEMU", "/dev/net/tun",
@@ -84,39 +105,24 @@ int virHostValidateQEMU(void)
                                     _("Load the 'tun' module to enable networking for QEMU guests")) < 0)
         ret = -1;
 
-    if (virHostValidateCGroupController("QEMU", "memory",
-                                        VIR_HOST_VALIDATE_WARN,
-                                        "MEMCG") < 0)
-        ret = -1;
-
-    if (virHostValidateCGroupController("QEMU", "cpu",
-                                        VIR_HOST_VALIDATE_WARN,
-                                        "CGROUP_CPU") < 0)
-        ret = -1;
-
-    if (virHostValidateCGroupController("QEMU", "cpuacct",
-                                        VIR_HOST_VALIDATE_WARN,
-                                        "CGROUP_CPUACCT") < 0)
-        ret = -1;
-
-    if (virHostValidateCGroupController("QEMU", "cpuset",
-                                        VIR_HOST_VALIDATE_WARN,
-                                        "CPUSETS") < 0)
-        ret = -1;
-
-    if (virHostValidateCGroupController("QEMU", "devices",
-                                        VIR_HOST_VALIDATE_WARN,
-                                        "CGROUP_DEVICES") < 0)
-        ret = -1;
-
-    if (virHostValidateCGroupController("QEMU", "blkio",
-                                        VIR_HOST_VALIDATE_WARN,
-                                        "BLK_CGROUP") < 0)
+    if (virHostValidateCGroupControllers("QEMU",
+                                         (1 << VIR_CGROUP_CONTROLLER_MEMORY) |
+                                         (1 << VIR_CGROUP_CONTROLLER_CPU) |
+                                         (1 << VIR_CGROUP_CONTROLLER_CPUACCT) |
+                                         (1 << VIR_CGROUP_CONTROLLER_CPUSET) |
+                                         (1 << VIR_CGROUP_CONTROLLER_DEVICES) |
+                                         (1 << VIR_CGROUP_CONTROLLER_BLKIO),
+                                         VIR_HOST_VALIDATE_WARN) < 0) {
         ret = -1;
+    }
 
     if (virHostValidateIOMMU("QEMU",
                              VIR_HOST_VALIDATE_WARN) < 0)
         ret = -1;
 
+    if (virHostValidateSecureGuests("QEMU",
+                                    VIR_HOST_VALIDATE_WARN) < 0)
+        ret = -1;
+
     return ret;
 }
